#*****************************************************************************
#*****************************************************************************
# R code for reading, checking, and computing length/waviness/angle phenotype
# data from raw primary root tracing, for the RNAseq June2013 seedling root
# traces produced using ImageJ.
#
# The code below in R language is run to read one or more tab-separated data
# files of primary root traces made with ImageJ and then save the resultant
# data to an Excel file.
#
# EDIT FILE Include_setAnalysisAndDate.R BEFORE RUNNING THIS CODE.
#*****************************************************************************
#*****************************************************************************

{

################################################################################
# Set working directory.
################################################################################

setwd("/Users/tedtoal/Documents/UCDavis/BradyLab/GrowthExperiments/Tomato/RootRNAseqJun2013_Phenotypes")

################################################################################
# Include other files used by this.
################################################################################

source("Include_setAnalysisAndDate.R")
source("Include_general.R")
source("Include_xlsx.R")
source("Include_plateLayout.R")
source("Include_dataColumnTypes.R")

################################################################################
# Set constants controlling the processing of the data.
################################################################################

# Names of the tab-separated data input files, produced using ImageJ, located in
# the rawInputDir directory.  There can be more than one file listed (separated by
# commas inside the list()), and if so, the data in all the files is read and used
# together.  Each list element consists of a 2-element character vector where the
# first element is the name of the person measuring the roots, and the second
# element is the name of the file.  The names of the people doing the measurements
# are recorded in column "measurer".
inputFiles = list(c("Livingstone", "Livingstone/roots.tsv"),
    c("Siobhan", "SiobhanMeasurements/Results_01_15_15.tsv"),
    c("Siobhan", "SiobhanMeasurements/Results_12_27_14.tsv"),
    c("Siobhan", "SiobhanMeasurements/Results_12_28_14.tsv"),
    c("Siobhan", "SiobhanMeasurements/Results_12_29_14.tsv"),
    c("Siobhan", "SiobhanMeasurements/Results_1_1_15.tsv"),
    c("Siobhan", "SiobhanMeasurements/Results_XMAS_2014.tsv"),
    c("Siobhan", "SiobhanMeasurements/Siobhan_Results_11_27.tsv"),
    c("Siobhan", "SiobhanMeasurements/Siobhan_Results_11_30.tsv"),
    c("Siobhan", "SiobhanMeasurements/Siobhan_Results_11_30_2.tsv"),
    c("Siobhan", "SiobhanMeasurements/Siobhan_Results_12_01.tsv"),
    c("Siobhan", "SiobhanMeasurements/Siobhan_Results_12_02.tsv"))

################################################################################
# Set constants used to control the verification of the data.
################################################################################

# Significant digits to retain in computed phenotype values.
sigDigits = 3

# Length to move back from tip of root to find tip angle.
tipAngLen.cm = 0.025

# The names of the data table columns, in order, as they appear in ALL input data
# files.  Do not include column "measurer", which is added separately.
commonInputDataCols = c("fileDate", "plateDate", "replicate", "genotype",
    "platePos", "plateLetter", "seed", "germDay", "collide", "along", "comment",
    "experiment", "cM", "scale", "plateRect", "pts")

# You may change the experimental design and either add additional columns to the
# data table, or remove some columns.  If you make this change after you've already
# processed some samples through the pipeline, you may be faced with the task of
# adding or removing columns from existing data files.  For example, you might add
# an "experNum" column to keep track of experiment number when you start working on
# experiment #2, and you may need to add that column to the data tables from the
# first experiment, putting "1" in those columns.  The following settings provide
# a way to automatically add and delete columns like this.
#   addCols is a list of column names that might be missing from some of the input
#   data files, and if so, you would like them to be added.  The list element name
#   is the name of the column to be added, and the list element value is the value
#   to place in that new column.  The value can be a simple value, or, if a more
#   complex expression is needed, a function can be used which is given the entire
#   data frame that lacks the desired column, and it must return the new column.
#   An example is given to show how this works: column plateRect will be added and
#   will have one of two different values depending on the value in column "plate".
#   Simple change this to an empty list list() if there are no columns that need to
#   be added to any of the data files.
addCols = list() # Simple version, no columns to be added.
#addCols = list(experiment="EXP1",  # Complex example, add columns "experiment" and "plateRect".
#    plateRect=function(df)
#        {
#        plateRect = rep("21,0,2920,2880", nrow(df))     # Plate A gets this for plateRect
#        plateRect[df$plate == "B"] = "0,2867,2949,3040" # Plate B gets this for plateRect
#        return(plateRect)
#        })

# Vector of column names that might be present in some of the input data files,
# and if so, you would like them to be deleted.  The values in this vector are
# the names of columns to be deleted if present.  Use an empty vector c() if
# there are no such columns.
deleteCols = c()
# Vector of names of the data table columns, in order, as they are to appear
# AFTER columns specified by addCols have been added and columns specified by
# deleteCols have been deleted.
revisedDataCols = c("fileDate", "plateDate", "measurer", "replicate", "genotype",
    "platePos", "plateLetter", "seed", "germDay", "collide", "along", "comment",
    "experiment", "cM", "scale", "plateRect", "pts")

# Information about how to sort the data table.  This information is for function
# order.special(), and it is a list with four elements:
#       - cols: vector of names of data table columns to use as sorting keys; the
#           first name is sorted first, and ties are broken by the second name, etc.
#       - lead0: vector of those names in "cols", if any, which are to be analyzed
#           for strings of digits that are converted to longer strings by adding
#           leading 0's, so that all digit strings in that argument are the same
#           length, causing e.g. IL10-1 to sort AFTER IL2-1 (it sorts as IL02-1).
#       - first: a list of vectors, each vector giving values in a "cols"-named
#           data column that are to appear FIRST in sorted order.  Each vector
#           in the list must have a NAME that matches a "cols" name, and the
#           elements must be values found in that data column that are to appear
#           FIRST in sorted order.
#       - last: like "first", but this determines values in "cols" columns that
#           are to appear LAST in order.
# Any of lead0, first, or last that are EMPTY must be specified as NULL.
sortCols = list(
    cols=c("genotype", "experiment", "replicate", "plateLetter", "seed"),
    lead0=c("genotype"),
    first=list(genotype=c("M82", "PENN")),
    last=NULL)

# Some data columns may contain text that the user may have entered in either upper
# or lower case, but which should be indistinguishable based on case.  For these
# columns it is best to force them to all upper or all lower case to avoid problems.
# Set these vectors to the names of columns to be forced to all-upper or all-lower
# case.
forceUpperCols = c("fileDate", "plateDate", "replicate", "genotype", "platePos",
    "plateLetter", "collide", "along", "experiment")
forceLowerCols = c()

# You may find that some columns had incorrect values entered into them that should
# be changed to some other value.  This list specifies values within columns that
# are to be changed to a new value.  Each list element is a sublist whose name
# is the column name to be changed and whose elements are 2- or 3-element vectors
# whose first element is the old value and whose second element is the new value to
# replace it, and whose third element is an optional TRUE/FALSE value which if TRUE
# means that the first element is actually an R regular expression (RE) to search the
# column for, and the second element is actually an R replacement expression (repRE)
# to generate the replacement.  In this latter case where regular expressions are used,
# the elements to be changed are found using grepl(RE,), and the replacement is done
# using sub(RE, repRE,).  Use an empty list 'list()' if there are no such changes required.
# See example for format.
#changeColValues = list(
#    side=list(c("LEFT","L"), c("RIGHT","R")),
#    type=list(c("COL","COL0")), # Change columns side and type, change "LEFT" to "L, "RIGHT" to "R", and "COL" to "COL0".
#    genotype=list(c("^1L", "IL", TRUE))) # Sometimes people use 1 instead of capital I in "IL".
#changeColValues = list() # No changes to be made.
changeColValues = list(
    # Change values in "replicate" and "experiment" columns for consistency.
    replicate=list(c("2","REP2"), c("3","REP3"), c("4","REP4"), c("5","REP5")),
    experiment=list(c("2","REP2"), c("3","REP3"), c("4","REP4"), c("5","REP5"),
        c("X1","REPX1"), c("X2","REPX2"), c("X3","REPX3"), c("X4","REPX4"), c("X5","REPX5"), c("X6","REPX6"),
        c("2_NEW9","REPN2"), c("3_NEW9","REPN3"), c("4_NEW9","REPN4"), c("5_NEW9","REPN5")),
    genotype=list(c("^1L", "IL", TRUE)), # Sometimes people use 1 instead of capital I in "IL".
    fileDate=list(c("^0([0-9])", "\\1", TRUE), c("^([0-9]+/)0", "\\1", TRUE)),  # Remove leading zeroes.
    plateDate=list(c("^0([0-9])", "\\1", TRUE), c("^([0-9]+/)0", "\\1", TRUE))) # Remove leading zeroes.

# Some data columns might have NA's that need to be changed to some other value.
# This vector specifies new values for NA's in columns whose name is given by
# the vector element name.
changeNAcols = c(replicate="UNKN")

# Some data columns might be required to never contain an NA value, and if they
# do, you may want an error message about this.  List the names of such columns
# here.
noNAcols = c("fileDate", "plateDate", "measurer", "genotype", "plateLetter", "seed",
    "collide", "along", "experiment", "cM", "scale", "plateRect")

# Similarly to above, some data columns might be required to use an empty string
# instead of NA.  List the names of columns where NA is to be changed to "".
blankNAcols = c("comment")

# Sometimes extraneous space characters make their way into data columns, and
# this can be a problem.  List the names of any columns that should never contain
# any space characters.  Space characters will be removed from these columns.
noSpacesCols = c("fileDate", "plateDate", "measurer", "replicate", "genotype",
    "platePos", "plateLetter", "seed", "germDay", "collide", "along", "cM", "scale", "plateRect", "pts")

# Some data columns might be required to never contain an empty string "" value,
# and if they do, you may want an error message about this.  List the names of
# such columns here.
noEmptyCols = c("fileDate", "plateDate", "measurer", "replicate", "genotype",
    "platePos", "plateLetter", "seed", "germDay", "collide", "along", "experiment",
    "cM", "scale", "plateRect", "pts")

# Some data columns should only contain integers, i.e. numbers consisting of 0-9
# with optional + and - signs but no decimal points or exponents.  List the names
# of such columns here.
integerCols = c("seed", "germDay")

# Some data columns should only contain lists of integers separated by commas or
# other characters.  List the names of such columns here in a list whose names
# are the column names and whose values are a sublist with the first element being
# either a single character that separates the data list elements or a REGULAR
# EXPRESSION that matches the separator characters, and the second element being
# the number of integers the list should contain, or NA if there is no specific
# number.
integerListCols = list(plateRect=list(",", 4))

# Some data columns should only contain floating point numbers, i.e. numbers that
# include decimal points and/or exponents.  List the names of such columns here.
numberCols = c("cM", "scale")

# Some data columns should only contain lists of floating point numbers separated
# by commas or other characters.  List the names of such columns here in a list
# whose names are the column names and whose values are a sublist with the first
# element being either a single character that separates the list elements or a
# REGULAR EXPRESSION that matches the separator characters, and the second element
# being the number of integers the list should contain, or NA if there is no
# specific number.
numberListCols = list(pts=list(",|;", NA))

# Some data columns should only contain logical values that are either TRUE or
# FALSE, which may be represented as T or F, or Y or N, or YES or NO also, and
# perhaps NA.  List the names of such columns here.  These columns are changed
# to contain either TRUE or FALSE.
logicalCols = c("collide", "along")

# Some data columns should contain numbers that are no smaller than some minimum
# value.  List the names of such columns, and their minimum allowed value, here.
# If any such column contains a smaller value, an error message is displayed.
minValueCols = c(seed=1, germDay=0, cM=1, scale=200)

# Some data columns should contain numbers that are no larger than some maximum
# value.  List the names of such columns, and their maximum allowed value, here.
# If any such column contains a larger value, an error message is displayed.
maxValueCols = c(seed=7, germDay=7, cM=50, scale=20000)

# Some data columns may have been read as numbers when actually they should be
# character strings, and in the process leading zeroes may have been removed that
# you wish to keep.  An example is a TDNA column containing TDNA line numbers such
# as 008357, which may have become "8357".  To restore such columns to their
# desired length, add entries to this vector to give the column name and desired
# number of digits.  The example here shows 6 digits for the "tdna" column.  If
# no columns like this exist, make this vector empty: c()  Note that if a TDNA
# line number ends with "C", the it won't be interpreted as a number and so any
# leading 0's won't have been stripped in the first place.
restoreLeadingZeroes = c()

# Some data columns may contain categorical data that takes on one of just a few
# possible values.  For example, a column might contain a limited number of
# genotypes, such as "M82" or "PENN".  List the names and allowable values for
# such columns here.  Each element of the list is one such column, with the
# element name being the column name and the element value being a vector of
# the allowable values in that column.
categoricalCols = list(
    replicate=c("REP2", "REP3", "REP4", "REP5", "UNKN"),
    plateLetter=c("A", "B", "C", "D", "E", "F"),
    experiment=c("REP2", "REP3", "REP4", "REP5",
        "REPX1", "REPX2", "REPX3", "REPX4", "REPX5", "REPX6",
        "REPN2", "REPN3", "REPN4", "REPN5"))

# Some columns are constant for all seeds on a plate.  Double check that this is
# true.  List here all column names whose values should be the same on a single
# plate.
plateConstantCols = c("fileDate", "plateDate", "measurer", "replicate", "genotype",
    "platePos", "plateLetter", "experiment")

# Print a warning message if the last two points at root tip have abs(dx)+abs(dy)
# less than this amount of pixels.
minLast2Pts.pixels = 8

################################################################################
# Set constants related to addition of synthetic data columns.
################################################################################

# This code requires that each plate have a unique ID.  If there is already a
# data table column giving a unique plate ID, it is specified here.  Otherwise,
# it must be possible to concatenate multiple data table columns to form an ID
# for a plate that is unique for each plate.  Specify a template string for the
# plate ID here, with data column names surrounded by % characters.  These will
# be replaced by the value in those columns (or the one column that IS the plate
# ID) to form a new string that is placed in a new column whose name is the name
# of the string in this vector.
plateIDtemplate = c(plateID="%genotype%_%plateLetter%_%experiment%")

# This code also requires that each seed have a unique ID.  This is defined in
# the same way as plateIDtemplate above.  The resulting ID must be unique for
# each seed.
seedIDtemplate = c(seedID="%genotype%_%experiment%_%plateLetter%%seed%")

# Additional synthetic data columns can be created by concatenating data in other
# columns.  Specify such columns here, in the same manner as for plateIDtemplate
# and seedIDtemplate above.  The vector element names are used as the new column
# names.  In particular, it is advisable to create synthetic columns for any nested
# factors that may be tested using linear models.
otherTemplates = c(
    # exp.gt="%experiment%.%genotype%",                   # This is interaction term experiment:genotype, needed sometimes.
    NULL) # NULL is not included in a vector.

################################################################################
# Read all raw data files in the input directory.
################################################################################

{
cat("Reading input data files:\n")
rawData = NULL
N = 0
for (V in inputFiles)
    {
    measurer = V[1]
    filename = V[2]
    cat("   ", filename, "\n")
    inFile = paste(rawInputDir, filename, sep=PATHSEP)
    df = read.table(inFile, header=TRUE, row.names=1, sep="\t", stringsAsFactors=FALSE)
    if (nrow(df) == 0)
        stop("No data was read from file: ", filename)
    # Make sure the common data columns expected to be present in all data files
    # actually are present.
    missingCols = commonInputDataCols[!commonInputDataCols %in% colnames(df)]
    if (length(missingCols) > 0)
        stop("These data columns were expected but not found: ", paste(missingCols, collapse=" "))
    # Add column "measurer".
    df$measurer = measurer
    # Remove unwanted data columns.
    rmvCols = deleteCols[deleteCols %in% colnames(df)]
    if (length(rmvCols) > 0)
        {
        df = df[, !colnames(df) %in% rmvCols]
        cat("      Removed unwanted columns: ", paste(rmvCols, collapse=" "), "\n")
        }
    # Add desired columns that are missing.
    newCols = names(addCols)[!names(addCols) %in% colnames(df)]
    if (length(newCols) > 0)
        {
        for (newCol in newCols)
            {
            newColValue = addCols[[newCol]]
            if (class(newColValue) == "function")
                newColValue = newColValue(df)
            df[, newCol] = newColValue
            if (length(newColValue) == 1)
                cat(paste("      Added new column: ", newCol, " = ", newColValue, sep="", collapse=" "), "\n")
            else
                cat(paste("      Added new column: ", newCol, " = (multiple values)", sep="", collapse=" "), "\n")
            }
        }
    # Now make sure that columns given by revisedDataCols are present, and that
    # revisedDataCols has not been mis-specified.
    if (any(!colnames(df) %in% revisedDataCols) || any(!revisedDataCols %in% colnames(df)))
        {
        expectedCols = revisedDataCols[!revisedDataCols %in% colnames(df)]
        unexpectedCols = colnames(df)[!colnames(df) %in% revisedDataCols]
        if (length(unexpectedCols) == 0)
            unexpectedCols = "(none)"
        stop("Expected data columns: ", paste(expectedCols, collapse=" "),
            "\nbut in their stead found columns: ", paste(unexpectedCols, collapse=" "))
        }
    # Add one more column as the first column: filename.
    df$filename = filename
    # Put the columns in the desired order.
    df = df[, c("filename", revisedDataCols)]
    rawData = rbind(rawData, df)
    N = N + 1
    }
cat("Finished reading ", N, " data files containing a total of ", nrow(rawData), " seed entries.\n", sep="")
}

# Take a look at the start of the raw data.
#head(rawData, 10)

################################################################################
# Perform verification of the data that was read.
################################################################################

# Get rid of leading and trailing spaces, if any, in the data.
rawData = gdata::trim(rawData)

# Force columns to upper and lower case as specified above.
for (col in forceUpperCols)
    rawData[, col] = toupper(rawData[, col])
for (col in forceLowerCols)
    rawData[, col] = tolower(rawData[, col])

# Change column values, if any are to be changed.
for (col in names(changeColValues))
    for (V in changeColValues[[col]])
        {
        if (length(V) < 3 || V[3] == "F" || V[3] == "FALSE")
            {
            oldVal = V[1]
            newVal = V[2]
            rawData[(!is.na(rawData[, col]) & rawData[, col] == oldVal), col] = newVal
            }
        else
            {
            RE = V[1]
            repRE = V[2]
            badRows = grepl(RE, rawData[, col])
            for (i in which(badRows))
                rawData[i, col] = sub(RE, repRE, rawData[i, col])
            }
        }

# Change NA to other values in specified columns.
for (col in names(changeNAcols))
    rawData[is.na(rawData[, col]), col] = changeNAcols[col]

# Test for NA in columns that should not have it.
for (col in noNAcols)
    {
    badRows = is.na(rawData[, col])
    if (any(badRows))
        stop("NA found in column '", col, "', ", getFileSeedStr(rawData, badRows, seedIDtemplate),
            ", which should not contain NAs")
    }

# Change NA to "" in columns that require it.
for (col in blankNAcols)
    rawData[is.na(rawData[, col]), col] = ""

# Remove spaces from columns that should not have them.
for (col in noSpacesCols)
    rawData[, col] = gsub(" ", "", rawData[, col])

# Test for "" in columns that should not have it.
for (col in noEmptyCols)
    {
    badRows = !is.na(rawData[, col]) & (rawData[, col] == "")
    if (any(badRows))
        stop("Empty string found in column '", col, "', ", getFileSeedStr(rawData, badRows, seedIDtemplate),
            ", which should not contain empty strings")
    }

# Test for integers in columns that should have them.
for (col in integerCols)
    {
    badRows = !is.na(rawData[, col]) & !grepl("^[+\\-]?[0-9]+$", rawData[, col])
    if (any(badRows))
        stop("Non-integer found in integer column ", col, ", ", getFileSeedStr(rawData, badRows, seedIDtemplate))
    rawData[, col] = as.integer(rawData[, col])
    }

# Test for integer lists in columns that should have them.
for (col in names(integerListCols))
    {
    sep = integerListCols[[col]][[1]]
    N = integerListCols[[col]][[2]]
    L = strsplit(rawData[, col], sep)
    badRows = sapply(L, function(V) any(!is.na(V) & !grepl("^[+\\-]?[0-9]+$", V)))
    if (any(badRows))
        stop("Non-integer-list found in integer-list column ", col, ", ",
            getFileSeedStr(rawData, badRows, seedIDtemplate))
    if (!is.na(N))
        {
        lens = sapply(L, length)
        badRows = (lens != N)
        if (any(badRows))
            stop("Integer list expected to have ", N, " elements but it didn't, ",
                getFileSeedStr(rawData, badRows, seedIDtemplate))
        }
    }

# Test for numbers in columns that should have them.
for (col in numberCols)
    {
    badRows = !is.na(rawData[, col]) &
        !grepl("^[+\\-]?[0-9]+(\\.[0-9]*)?([eE][+\\-]?[0-9]+)?$", rawData[, col]) &
        !grepl("^[+\\-]?[0-9]*\\.[0-9]+([eE][+\\-]?[0-9]+)?$", rawData[, col])
    if (any(badRows))
        stop("Non-number found in number column ", col, ", ", getFileSeedStr(rawData, badRows, seedIDtemplate))
    rawData[, col] = as.numeric(rawData[, col])
    }

# Test for number lists in columns that should have them.
for (col in names(numberListCols))
    {
    sep = numberListCols[[col]][[1]]
    N = numberListCols[[col]][[2]]
    L = strsplit(rawData[, col], sep)
    badRows = sapply(L, function(V) any(!is.na(V) &
        !grepl("^[+\\-]?[0-9]+(\\.[0-9]*)?([eE][+\\-]?[0-9]+)?$", V) &
        !grepl("^[+\\-]?[0-9]*\\.[0-9]+([eE][+\\-]?[0-9]+)?$", V)))
    if (any(badRows))
        stop("Non-number-list found in number-list column ", col, ", ",
            getFileSeedStr(rawData, badRows, seedIDtemplate))
    if (!is.na(N))
        {
        lens = sapply(L, length)
        badRows = (lens != N)
        if (any(badRows))
            stop("Number list expected to have ", N, " elements but it didn't, ",
                getFileSeedStr(rawData, badRows, seedIDtemplate))
        }
    }

# Test for logical values in columns that should have them, and make sure they are TRUE or FALSE.
for (col in logicalCols)
    {
    badRows = !is.na(rawData[, col]) & !grepl("TRUE|FALSE|T|F|YES|NO|Y|N", rawData[, col])
    if (any(badRows))
        stop("Non-logical value found in logical column ", col, ", ",
            getFileSeedStr(rawData, badRows, seedIDtemplate))
    rawData[!is.na(rawData[, col]), col] = grepl("TRUE|T|YES|Y", rawData[, col])
    rawData[, col] = as.logical(rawData[, col])
    }

# Test for numbers that are too small.
for (col in names(minValueCols))
    {
    badRows = !is.na(rawData[, col]) & (rawData[, col] < minValueCols[col])
    if (any(badRows))
        stop("Too-small number found in column '", col, "', ", getFileSeedStr(rawData, badRows, seedIDtemplate))
    }

# Test for numbers that are too large.
for (col in names(maxValueCols))
    {
    badRows = !is.na(rawData[, col]) & (rawData[, col] > maxValueCols[col])
    if (any(badRows))
        stop("Too-large number found in column '", col, "', ", getFileSeedStr(rawData, badRows, seedIDtemplate))
    }

# Sort the data for output, but continue using unsorted data for checking data integrity below.
L = as.list(rawData[, sortCols$cols])
L = c(L, lead0=list(sortCols$lead0), first=list(sortCols$first), last=list(sortCols$last))
df = rawData[do.call(order.special, L),]
#head(df$genotype, 300)
#tail(df$genotype, 300)

# Create the output directory if it doesn't exist.
dir.create(dataDir, showWarnings=FALSE)

# Write the raw data to a .xlsx file.
make.xlsx(df, cleanedRawDataPath, freezeTopRow=TRUE, halignDefault="CENTER")

# Restore leading zeroes in some columns.
for (col in names(restoreLeadingZeroes))
    rawData[, col] = fillVec(rawData[, col], restoreLeadingZeroes[col], "0")

# Check for invalid values in categorical columns.
for (col in names(categoricalCols))
    {
    badRows = !rawData[, col] %in% categoricalCols[[col]]
    if (any(badRows))
        stop("Unexpected value(s) in column '", col, "', ", getFileSeedStr(rawData, badRows, seedIDtemplate),
            ",\nThat column contains these categorical values: ", paste(unique(rawData[, col]), collapse=","))
    }

################################################################################
# Create synthetic columns using the defined template strings.
################################################################################

synCols = c(plateIDtemplate, seedIDtemplate, otherTemplates)
for (col in names(synCols))
    rawData[,col] = replaceTemplateString(synCols[col], rawData)

################################################################################
# Perform some consistency checks on the data.
################################################################################

# Check that columns that should be constant on the same plate are indeed so.
Nerrs = 0
for (col in plateConstantCols)
    {
    badRows = tapply(rawData[, col], rawData$plateID, function(V) !all(is.na(V) | V == V[1]))
    if (any(badRows))
        {
        for (i in which(badRows))
            {
            plateID = names(badRows)[i]
            cat("Not-constant value(s) in column ", col, " ", plateID, rawData[rawData$plateID == plateID, "filename"][1], "\n")
            Nerrs = Nerrs + 1
            }
        }
    }
if (Nerrs > 0)
    stop(Nerrs, " errors with plate-constant columns")

# If sided or interleave, there should be two seed types on each plate, and the
# seed numbers should occur in two sets, such that one set has one type and the
# other set has the other type of seed.  Even missing seeds should still have an
# entry, so we always have exactly two of each seed number on each plate.  Make
# sure this is true.
if (sided || interleave)
    {
    halfSeedsPerPlate = numSeedsPerPlate/2
    expectedSeedNums = 1:halfSeedsPerPlate
    seedNums = tapply(rawData$seed, list(rawData$plateID, rawData$type), function(x) all(sort(x) == expectedSeedNums))
    goodPlates = apply(seedNums, 1, all)
    if (any(!goodPlates))
        stop("Some plates do not have seeds 1..", halfSeedsPerPlate, " for each seed type ",
            paste(colnames(seedNums), collapse=","), ": ", getPrint(names(goodPlates)[!goodPlates]))
    }

# Get the mean scale factor to see if all data rows seem to be the same scale.
# They should all be at the same scale.  Also, round the scale value to the nearest
# integer, it will still have 4 significant digits.
rawData$scale = round(rawData$scale)
scale = rawData$cM/rawData$scale
if (any(is.na(scale)))
    stop("Some data has NA for its cM and/or scale entries.")
mean.scale = mean(scale)
SD.scale = sd(scale)
CV.scale = SD.scale/mean.scale
if (CV.scale > 0.01)
    {
    stop("Coef. of variation of the scale factor across seeds is rather large: ", CV.scale,
        "\nThese plate IDs have scale factor exceeding +/- 1% of the mean:\n",
        getPrint(unique(rawData$plateID[abs(scale-mean.scale)/mean.scale > 0.01]), "    "))
    }
cmPerPixel = mean.scale

# Check that the size of the plate rectangle is within 5% of the known plate size in all cases.
plateSize.pixels.x = sapply(rawData$plateRect, function(S) as.integer(unlist(strsplit(S, ",", fixed=TRUE))[3]))
names(plateSize.pixels.x) = NULL
plateSize.pixels.y = sapply(rawData$plateRect, function(S) as.integer(unlist(strsplit(S, ",", fixed=TRUE))[4]))
names(plateSize.pixels.y) = NULL
plateSize.cm.x = plateSize.pixels.x * cmPerPixel
plateSize.cm.y = plateSize.pixels.y * cmPerPixel
maxDeviationPct = 6
deviation.x = round(100*(plateSize.cm.x-plateSize.cm)/plateSize.cm, 1)
deviation.y = round(100*(plateSize.cm.y-plateSize.cm)/plateSize.cm, 1)
deviants.x = (abs(deviation.x) > maxDeviationPct)
deviants.y = (abs(deviation.y) > maxDeviationPct)
deviants = deviants.x | deviants.y
if (any(deviants))
    {
    stop("Some plate rectangles are more than ", maxDeviationPct, "% different than actual plate size of ", plateSize.cm, " cm:\n",
        getPrint(unique(paste(rawData$plateID[deviants], " (", deviation.x[deviants], "%,", deviation.y[deviants], "%)", sep="")), "    "))
    }

# It is very annoying that the seed data does not include the seed row and column
# number.  I should have added that to the data; it is in fact available in the
# macro code and is easily added.  But now there's a lot of data with no seed row
# or column.
# Check that all seeds have the top of their root somewhere near where the seed
# should have been planted.
#***********

################################################################################
# Compute the phenotype values from the raw data.
################################################################################

# Loop for each seedling and compute its lengths, waviness, and angles.
# Two lengths are computed:
#   "straightLen" is the length of a line from the first to the last point on
#       the root.
#   "rootLen" is the length along the root axis.
# Both lengths are in cM.
# Waviness is computed:
#   "waviness" is rootLen/straightLen - 1.  0=perfectly straight, >0=wavy.  
#           This doesn't seem to produce results that matches my intuitive feel
#           of amount of waviness.  However, I think it matches the way Wolfgang
#           measured it.  Their description: index of waviness: pixels of primary
#           root skeleton divided by root length vector (shortest distance from
#           hypocotyl/root boundary to the root tip).			
#   Other methods tried:
#       2. "waviness" is sum of absolute value of the angle of change of root
#           polyline segment direction, divided by rootLen.  Units are degrees
#           per cM.
#       3. "waviness" is sum of (absolute value of the difference in orientation
#           angles of adjacent pairs of root polyline segments times mean length
#           of the two segments), divided by rootLen.  Units are degrees.
# These angles are computed:
#   "tipAng" is the angle that the last line segment tracing the root axis makes
#       with the gravity vector.
#   "avgAbsAng" is the average along the entire root of the absolute value of
#       the angle of the root with the gravity vector.
{
if (analysisVariant == "Orig")
    {
#   "wholeAng" is the angle that the line from the first to last point makes
#       with the gravity vector.
#   "quarterAng" is the angle made by the line segment from the root tip through
#       the point lying one-quarter of rootLen back towards the root start.
#   "tenthAng" is the angle made by the line segment from the root tip through
#       the point lying one-tenth of rootLen back towards the root start.
    }
else if (analysisVariant == "thirdsAngs" || analysisVariant == "thirdsAngB")
    {
#   "first3rdAng" is the average along the first third of the root of the absolute
#       value of the angle of the root with the gravity vector.
#   "mid3rdAng" is the average along the middle third of the root of the absolute
#       value of the angle of the root with the gravity vector.
#   "last3rdAng" is the average along the last third of the root of the absolute
#       value of the angle of the root with the gravity vector.
    }
}
# All angles are in degrees, 0 being straight down, negative being clockwise
# from straight down and positive being counter-clockwise from straight down.
# However, since our analysis will use only the absolute value of the angles,
# the ABOVE ANGLE MEASURES ARE ALL IN THE RANGE [0,+180].
# These angle signs are computed:
#   "tipAngSign" is +1 if tipAng > 0, -1 if tipAng < 0, 0 if exactly 0.
#   "avgAngSign" is +1 if the average of the angle direction along the entire
#       root is > 0, -1 if < 0.
{
if (analysisVariant == "Orig")
    {
#   "wholeAngSign" is +1 if wholeAng > 0, -1 if wholeAng < 0, 0 if exactly 0.
#   "quarterAngSign" is +1 if quarterAng > 0, -1 if quarterAng < 0, 0 if exactly 0.
#   "tenthAngSign" is +1 if tenthAng > 0, -1 if tenthAng < 0, 0 if exactly 0.
    }
else if (analysisVariant == "thirdsAngs" || analysisVariant == "thirdsAngB")
    {
#   "first3rdAngSign" is +1 if the average of the angle direction along the first
#       third of the root is > 0, -1 if < 0.
#   "mid3rdAngSign" is +1 if the average of the angle direction along the middle
#       third of the root is > 0, -1 if < 0.
#   "last3rdAngSign" is +1 if the average of the angle direction along the last
#       third of the root is > 0, -1 if < 0.
    }
}

# Compute the angle of a line segment in degrees, given the dx and dy values
# from the start point of the segment to the end point of the segment.  Right
# is +dx, down is +dy.  Return the angle in degrees, negative for clockwise from
# straight down (-180,+180].  If dx and dy are vectors (of the same size), a
# vector of angles of that size is returned.
segAng = function(dx, dy)
    {
    # Negate dy so our Y-coordinates are the more usual variety, with +Y up.
    dy = -dy
    # Straight down would have some negative value for dy and would have dx = 0,
    # giving an ang of -pi/2 radians, or 0 after conversion to degrees and adding
    # 90 degrees.
    ang = atan2(dy, dx)*degPerRad + 90
    # The range of atan2 is [-pi,+pi], so after conversion to degrees the range
    # is [-180,+180], and after adding 90 degrees, the range is [-90,+270].  We
    # need to move (+180,+270] to (-180,-90], which we do by subtracting 360
    # degrees.
    ang[ang > +180] = ang[ang > +180] - 360
    # And do this to handle roundoff errors in the negative angle direction.
    ang[ang > -181 & ang < -180] = -180
    if (any(ang > +180))
        stop("Angle error (+).")
    if (any(ang < -180))
        stop("Angle error (-).")
    return(ang)
    }

# Given a polyline (group of connected line segments), compute the angle of
# the line segment whose endpoint is the endpoint of the polyline, and whose
# startpoint is the point located at a specified distance along the polyline
# from its endpoint.
# Arguments:
#   len: the distance from the polyline endpoint at which the angle is to be measured.
#   revSegLens: reversed vector of lengths of each polyline segment, first vector
#       element is the length of the last polyline segment.
#   dxRev: reversed vector of x-position differences along polyline segments,
#       first vector element is x-difference of start and end point of last
#       polyline difference (positive is to right).
#   dyRev: like dxRev but for y-position (positive is down).
# Returns the angle in degrees, negative for clockwise from straight down, (-180,+180].
computeAngleOfLastPortionOfPolyline = function(len, revSegLens, dxRev, dyRev)
    {
    # Compute cumulative length of each polyline point from the endpoint.
    cumRevSegLens = c(0, cumsum(revSegLens))
    # Get lastFull = index in cumRevSegLens of last (highest) segment completely
    # included by len = index in revSegLens, dxRev, dyRev of next segment after it.
    lastFull = match(TRUE, cumRevSegLens > len) - 1
    # How much does len extend partially into segment following lastFull segment?
    amtLastSeg = len - cumRevSegLens[lastFull]
    # How long is that last segment?
    lenLastSeg = revSegLens[lastFull]
    # What is the fractional extension into it?
    fracLastSeg = amtLastSeg/lenLastSeg
    # Find dx and dy from polyline endpoint to len position.
    dxLastSeg = fracLastSeg * dxRev[lastFull] 
    dyLastSeg = fracLastSeg * dyRev[lastFull]
    dxLastPortion = sum(dxRev[1:(lastFull-1)]) + dxLastSeg
    dyLastPortion = sum(dyRev[1:(lastFull-1)]) + dyLastSeg
    # Reverse these so direction is nearer start of the polyline towards its endpoint.
    # Right is +X, down is +Y.
    dxLastPortion = -dxLastPortion
    dyLastPortion = -dyLastPortion
    return(signif(segAng(dxLastPortion, dyLastPortion), sigDigits))
    }

# Given the lengths and angles of each segment of the root polyline, and the sum
# of those lengths, compute the average absolute angle along the portion of the
# root specified by startFrac and endFrac, along with the sign of the average
# angle.
# Arguments:
#   rootLen: length of root along root centerline.
#   startFrac: fraction (0..1) of rootLen from top of root at which to start the
#       angle computation.
#   endFrac: fraction (0..1) of rootLen from top of root at which to end the
#       angle computation.
#   segLens: vector of lengths of each root polyline segment, first vector
#       element is the length of the first polyline segment.
#   segAngs: vector of angles of each root polyline segment as returned by segAng,
#       first vector element is the angle of the first polyline segment.
# Returns a list with two elements:
#   ang: the angle in degrees [0,+180]
#   sgn: the sign (-1, 0, or +1).
computeAvgAbsAngle = function(rootLen, startFrac, endFrac, segLens, segAngs)
    {
    startLen = rootLen*startFrac
    endLen = rootLen*endFrac
    cumLen = c(0, cumsum(segLens))
    startSeg = match(TRUE, startLen < cumLen) - 1
    if (is.na(startSeg) || startSeg < 1 || startSeg > length(segLens)) stop("Error finding start segment of root")
    endSeg = match(TRUE, endLen <= cumLen) - 1
    if (is.na(endSeg) || endSeg < startSeg || endSeg > length(segLens)) stop("Error finding end segment of root")
    if (startSeg == endSeg)
        {
        segAng = segAngs[startSeg]
        ang = abs(segAng)
        sgn = sign(segAng)
        }
    else
        {
        startSegPartialLen = segLens[startSeg] - (startLen - cumLen[startSeg])
        if (startSegPartialLen < 0) stop("Error computing partial start seg len")
        endSegPartialLen = endLen - cumLen[endSeg]
        if (endSegPartialLen < 0) stop("Error computing partial end seg len")
        segAngs = segAngs[startSeg:endSeg]
        if (startSeg+1 == endSeg)
            segLens = c(startSegPartialLen, endSegPartialLen)
        else
            segLens = c(startSegPartialLen, segLens[(startSeg+1):(endSeg-1)], endSegPartialLen)
        totalLen = sum(segLens)
        ang = sum(abs(segAngs)*segLens)/totalLen
        # Note: the calculation of angle sign here was incorrect in the original analysis
        # used for writing paper.  However, signs are not being used in the paper.
        sgn = sign(sum(sign(segAngs)*segLens)/totalLen)
        }
    ang = signif(ang, sigDigits)
    return(list(ang=ang, sgn=sgn))
    }

# Copy rawData to new data frame dfRoots and initialize the computed values to NA.
dfRoots = rawData
dfRoots$straightLen = NA
dfRoots$rootLen = NA
dfRoots$waviness = NA
{
if (analysisVariant == "Orig")
    {
    dfRoots$wholeAng = NA
    dfRoots$quarterAng = NA
    dfRoots$tenthAng = NA
    dfRoots$tipAng = NA
    dfRoots$avgAbsAng = NA
    dfRoots$wholeAngSign = NA
    dfRoots$quarterAngSign = NA
    dfRoots$tenthAngSign = NA
    dfRoots$tipAngSign = NA
    dfRoots$avgAngSign = NA
    }
else if (analysisVariant == "thirdsAngs" || analysisVariant == "thirdsAngB")
    {
    dfRoots$tipAng = NA
    dfRoots$tipAngSign = NA
    dfRoots$avgAbsAng = NA
    dfRoots$avgAngSign = NA
    dfRoots$first3rdAng = NA
    dfRoots$first3rdAngSign = NA
    dfRoots$mid3rdAng = NA
    dfRoots$mid3rdAngSign = NA
    dfRoots$last3rdAng = NA
    dfRoots$last3rdAngSign = NA
    }
}

# Calculate the computed values and place them in dfRoots.
numWarnings = 0
cat("Calculating computed phenotype values.\n")
for (i in 1:nrow(dfRoots))
    {
    #cat("i=", i, "\n")
    xy = dfRoots$pts[i]
    if (is.na(xy))
        next
    # Get the points along the root.  The first point is the absolute pixel
    # position of the start of the root (top of root), and subsequent points
    # are the difference in pixel position from the previous point.  Change
    # the scale from pixels to cM.
    xy = unlist(strsplit(xy, ";", fixed=TRUE))
    x = as.numeric(sapply(xy, function(P) unlist(strsplit(P, ",", fixed=TRUE))[1]))
    y = as.numeric(sapply(xy, function(P) unlist(strsplit(P, ",", fixed=TRUE))[2]))
    N = length(x)
    dd = abs(x[N])+abs(y[N])
    if (dd < minLast2Pts.pixels)
        {
        cat("Warning: Last two points at tip of seed have summed values of only ",
            dd, ", ", getFileSeedStr(dfRoots, i, seedIDtemplate), "\n")
        numWarnings = numWarnings + 1
        }
    scale = dfRoots$cM[i]/dfRoots$scale[i]
    x = x * scale
    y = y * scale
    # Get the plate rectangle.
    plateRect = as.integer(unlist(strsplit(dfRoots$plateRect[i], ",", fixed=TRUE))) * scale
    plate.ul.x.cm = plateRect[1]
    plate.ul.y.cm = plateRect[2]
    plate.lr.x.cm = plateRect[1] + plateRect[3]
    plate.lr.y.cm = plateRect[2] + plateRect[4]
    # Check to see if all points are within the plate rectangle.
    xt = cumsum(x)
    yt = cumsum(y)
    if (any(xt < plate.ul.x.cm))
        {
        cat("Seed is to left of the plate rectangle by ", max(plate.ul.x.cm-xt), " cm, ",
            getFileSeedStr(dfRoots, i, seedIDtemplate), "\n")
        numWarnings = numWarnings + 1
        }
    if (any(yt < plate.ul.y.cm))
        {
        cat("Seed is above the plate rectangle by ", max(plate.ul.y.cm-yt), " cm, ",
            getFileSeedStr(dfRoots, i, seedIDtemplate), "\n")
        numWarnings = numWarnings + 1
        }
    if (any(xt > plate.lr.x.cm))
        {
        cat("Seed is to right of the plate rectangle by ", max(xt-plate.ul.x.cm), " cm, ",
            getFileSeedStr(dfRoots, i, seedIDtemplate), "\n")
        numWarnings = numWarnings + 1
        }
    if (any(yt > plate.lr.y.cm))
        {
        cat("Seed is below the plate rectangle by ", max(yt-plate.ul.y.cm), " cm, ",
            getFileSeedStr(dfRoots, i, seedIDtemplate), "\n")
        numWarnings = numWarnings + 1
        }

    # Compute straight-line length from start to end of root.
    dx = x[-1]
    dy = y[-1]
    dxSum = sum(dx)
    dySum = sum(dy)
    dfRoots$straightLen[i] = signif(sqrt(dxSum^2+dySum^2), sigDigits)

    # Compute length of each segment.  We simply need to compute length of
    # each delta position value.
    segLens = sqrt(dx^2+dy^2)
    rootLen = sum(segLens)
    dfRoots$rootLen[i] = signif(rootLen, sigDigits)

    # Compute waviness = rootLen/straightLen - 1
    dfRoots$waviness[i] = signif((dfRoots$rootLen[i]/dfRoots$straightLen[i]) - 1, sigDigits)

    # Compute waviness = sum(abs(delta segment angle * mean segment len)/rootLen
    #angs = atan2(-dy, dx)*degPerRad + 90  ### THIS atan2 is wrong, fix it if uncommenting this
    #dxAngs = angs[-1] - angs[-length(angs)]
    #meanSegLens = (segLens[-1] + segLens[-length(segLens)])/2
    #dfRoots$waviness[i] = signif(sum(abs(dxAngs*meanSegLens))/dfRoots$rootLen[i], sigDigits)

    # Computing average absolute angles requires that we have the angle of each segment.
    segAngs = segAng(dx, dy)

    # Computing some angles requires that we reverse segLens, dx, and dy.
    # Remember though that reversed dx and dy must also have reversed signs since
    # we use (upper point - lower point).  This does not change that +x is right,
    # +y is down.
    revSegLens = rev(segLens)
    dxRev = -rev(dx)
    dyRev = -rev(dy)

    # Compute tipAng = angle of very last bit of root, plus tipAngSign.
    if (dyRev[1] > 0 && dyRev[2] < 0)
        {
        cat("Seed goes upward at last point: ", getFileSeedStr(dfRoots, i, seedIDtemplate), "\n")
        numWarnings = numWarnings + 1
        }
    dfRoots$tipAng[i] = computeAngleOfLastPortionOfPolyline(tipAngLen.cm, revSegLens, dxRev, dyRev)
    dfRoots$tipAngSign[i] = sign(dfRoots$tipAng[i])
    dfRoots$tipAng[i] = abs(dfRoots$tipAng[i])

    # Compute avgAbsAng = average absolute angle along entire root, plus avgAngSign.
    L = computeAvgAbsAngle(rootLen, 0, 1, segLens, segAngs)
    dfRoots$avgAbsAng[i] = L$ang
    dfRoots$avgAngSign[i] = L$sgn

    # Compute other angles depending on setting of analysisVariant.
    if (analysisVariant == "Orig")
        {
        # Compute wholeAng = angle of line from start to end of root, plus wholeAngSign.
        # Remember that dy is actually negative, it goes downwards.  A negative
        # angle is clockwise of straight down.
        dfRoots$wholeAng[i] = signif(segAng(dxSum, dySum), sigDigits)
        dfRoots$wholeAngSign[i] = sign(dfRoots$wholeAng[i])
        dfRoots$wholeAng[i] = abs(dfRoots$wholeAng[i])

        # Compute quarterAng = angle of last quarter of root, plus quarterAngSign.
        quarterLen = dfRoots$rootLen[i]/4
        dfRoots$quarterAng[i] = computeAngleOfLastPortionOfPolyline(quarterLen, revSegLens, dxRev, dyRev)
        dfRoots$quarterAngSign[i] = sign(dfRoots$quarterAng[i])
        dfRoots$quarterAng[i] = abs(dfRoots$quarterAng[i])

        # Compute tenthAng = angle of very last tenth of root, plus tenthAngSign.
        tenthLen = dfRoots$rootLen[i]/10
        dfRoots$tenthAng[i] = computeAngleOfLastPortionOfPolyline(tenthLen, revSegLens, dxRev, dyRev)
        dfRoots$tenthAngSign[i] = sign(dfRoots$tenthAng[i])
        dfRoots$tenthAng[i] = abs(dfRoots$tenthAng[i])
        }
    else if (analysisVariant == "thirdsAngs" || analysisVariant == "thirdsAngB")
        {
        # first third
        L = computeAvgAbsAngle(rootLen, 0, 1/3, segLens, segAngs)
        dfRoots$first3rdAng[i] = L$ang
        dfRoots$first3rdAngSign[i] = L$sgn

        # middle third
        L = computeAvgAbsAngle(rootLen, 1/3, 2/3, segLens, segAngs)
        dfRoots$mid3rdAng[i] = L$ang
        dfRoots$mid3rdAngSign[i] = L$sgn

        # last third
        L = computeAvgAbsAngle(rootLen, 2/3, 1, segLens, segAngs)
        dfRoots$last3rdAng[i] = L$ang
        dfRoots$last3rdAngSign[i] = L$sgn
        }
    }
if (numWarnings > 0)
    stop("Error, warnings issued, pleased fix them")
cat("Finished calculating computed phenotype values.\n")

################################################################################
# Compute three ages for each seedling:
#   platingAge: days from plating until harvest.
#   germAge: days from plating until germination.
#   rootAge: days from germination until harvest.
################################################################################

# Compute column "platingAge", the age in days of each root at the time it was
# imaged and harvested.  The age is just the image file date from the "fileDate"
# column minus the plating date from the "plateDate" column.

plateDate = as.Date(dfRoots$plateDate, "%m/%d/%y")
fileDate = as.Date(dfRoots$fileDate, "%m/%d/%y")
platingAge = as.numeric(fileDate - plateDate, units="days")
badRows = (platingAge < 0)
if (any(badRows))
    stop("Bad plate or file date(s):", getFileSeedStr(dfRoots, badRows, seedIDtemplate))
#table(platingAge)

# Compute the age of each seed at the time it germinated, i.e. days from plating
# to germination, and add this as column "germAge".  The age is the germination
# day minus the plate day, but since germDay is given as a number from 1 to 7
# representing Sun-Sat, there can be complications.  Say a seed germinates on
# the 8th day after plating (can it?).  That would be the same as germinating
# 1 day after plating (can it?).

plateDay = weekdays(plateDate, abbreviate=TRUE)
germDayNums = c(Sun=1, Mon=2, Tue=3, Wed=4, Thu=5, Fri=6, Sat=7) # germination days marked 1 to 7 are these day names
plateDayNums = germDayNums[plateDay]
germAge = dfRoots$germDay - plateDayNums
#table(germAge)
# Negative numbers mean it was plated near day 7 and germinated after rollover to day 1.
germAge[!is.na(germAge) & germAge <= 0] = germAge[!is.na(germAge) & germAge <= 0] + 7
#table(germAge)

# Also compute the age of each root at the time it was imaged, as days after
# germination, and add this as column "rootAge".  The age is platingAge-germAge.
# Roots were harvested either 3 or 4 days after germination, so root age should
# be either 3 or 4.  However, some roots may have been traced that shouldn't
# have been (because they weren't harvested).  We will leave these roots in
# the data for now, but they need to be removed for modelling analysis.

rootAge = platingAge - germAge
#table(rootAge)

# Add the new columns after column "plateDate".
colnum = which(colnames(dfRoots) == "plateDate")
dfRoots = data.frame(dfRoots[, 1:colnum], platingAge=platingAge, germAge=germAge,
    rootAge=rootAge, dfRoots[(colnum+1):ncol(dfRoots)], stringsAsFactors=FALSE)

# Check some anomalies.
#dfRoots[!is.na(rootAge) & rootAge == 1,] # 3, all ok, they really did get imaged one day after germination.
#dfRoots[!is.na(rootAge) & rootAge == 6,] # Plates REPN5 IL6-2 C and D have wrong plating and file dates.  Fixed.
#dfRoots$germDay[dfRoots$plateID == "IL7-5-1_C_REPX3"] # Looking at plate and plotted roots, I see that germDay 2 was misentered, should be 6.  Fixed.
# Use plate scanning date to double-check.  The scanning date was the day the
# plate was opened for root harvest, which would have been either 3, 4, or 5
# days after germination of most roots on that plate.  All plates with a root
# that germinated 1 day after plating should have been harvested no later than
# about 6 days after plating.
#table(germAge)
#table(dfRoots$platingAge)
#dfRoots[!is.na(germAge) & germAge == 1 & dfRoots$platingAge > 6,]
#dfRoots[dfRoots$plateID == "IL5-1_A_REPX2",] # First three have incorrect dates.  Fixed.
# All plates with a root that germinated 2 days after plating should have been
# harvested no later than about 7 days after plating.
#dfRoots[!is.na(germAge) & germAge == 2 & dfRoots$platingAge > 7,]
# All plates with a root that germinated 7 days after plating should have been
# harvested AT LEAST 8 days after plating.
#dfRoots[!is.na(germAge) & germAge == 7 & dfRoots$platingAge < 8,]
# All plates with a root that germinated 6 days after plating should have been
# harvested AT LEAST 7 days after plating.
#dfRoots[!is.na(germAge) & germAge == 6 & dfRoots$platingAge < 7,]
# Looks good.

################################################################################
# Write out the new data file with phenotype computation columns added.
################################################################################

# Sort the data.
L = as.list(dfRoots[, sortCols$cols])
L = c(L, lead0=list(sortCols$lead0), first=list(sortCols$first), last=list(sortCols$last))
dfRoots = dfRoots[do.call(order.special, L),]
#head(dfRoots$genotype, 300)
#tail(df$Rootsgenotype, 300)

# Put columns in order given by processedDataColClasses, then save the dfRoots
# data frame in a file.
dfRoots = dfRoots[, names(processedDataColClasses)]
make.xlsx(dfRoots, processedDataPath, freezeTopRow=TRUE, halignDefault="CENTER")
# Re-read.
# dfRoots = read.xlsx2(processedDataPath, sheetIndex=1, header=TRUE, stringsAsFactors=FALSE)
cat("Finished writing processed data file.\n")

}
