################################################################################
# This file contains R code to analyze root phenotype data from the Brady Lab
# root RNAseq experiment using tomato introgression lines (ILs) with RNAseq
# data first obtained in June 2013.  This code reads measured and pre-processed
# phenotype data, including root lengths and various measures of root angle,
# and runs it through linear models to estimate significant phenotype effects
# in different ILs.
#
# EDIT FILE Include_setAnalysisAndDate.R BEFORE RUNNING THIS CODE.
################################################################################

# Set working directory.
setwd("~/Documents/UCDavis/BradyLab/GrowthExperiments/Tomato/RootRNAseqJun2013_Phenotypes")

################################################################################
# Include files.
################################################################################

source("Include_setAnalysisAndDate.R")
source("Include_general.R")
source("Include_xlsx.R")
source("Include_dataColumnTypes.R")
phenoRNAseqRootSep2011Dir = "~/Documents/UCDavis/BradyLab/CellCountingGravity"
source(paste(phenoRNAseqRootSep2011Dir, "Include_phenoRNAseqRootSep2011.R", sep=PATHSEP))

################################################################################
# Load libraries used by this.
################################################################################

library("nlme")
library("multcomp")

################################################################################
# Set basic constants controlling the analysis.
################################################################################

# Output file pathnames.
logFile = paste(analysisOutputPrefix, "logfile.txt", sep="")
traitsFile = paste(analysisOutputPrefix, "traits.tsv", sep="")
groupsFile = paste(analysisOutputPrefix, "traitGroups.tsv", sep="")
copyOfInputDataFile = paste(analysisDir, paste(dataFilePrefix, processedDataFile, sep=""), sep=PATHSEP)
vettedInputDataFile = paste(analysisOutputPrefix, "inputDataFiltered.tsv", sep="")
biorepMeansFile = paste(analysisOutputPrefix, "biorepMeans.tsv", sep="")
getPerGenotypeDataFilename = function(endName) return(paste(analysisOutputPrefix, endName, sep=""))

# Set this TRUE to remove all roots whose "collide" value is TRUE (root collided
# with another root).
rmvCollideRoots = FALSE

# Set this TRUE to remove all roots whose "along" value is TRUE (root grew
# alongside another root).
rmvAlongRoots = TRUE

# Set this to the minimum number of days old a root must be (after germination)
# to retain the root for analysis.
rmvRootsLessThanDAG = 3

# Set this to the maximum number of days old a root can be (after germination)
# to retain the root for analysis.
rmvRootsMoreThanDAG = 4

# Set this to a vector of traits to be removed and not analyzed.
{
if (analysisVariant == "Orig")
    rmvTraits = c("waviness", "wholeAngSign", "quarterAngSign", "tenthAngSign", "tipAngSign", "avgAngSign")
else if (analysisVariant == "thirdsAngs" || analysisVariant == "thirdsAngB")
    rmvTraits = c("waviness", "tipAngSign", "avgAngSign", "first3rdAngSign", "mid3rdAngSign", "last3rdAngSign")
else
    stop("Bad analysisVariant value")
}

# Minimum number of measurements of a trait in a genotype on a particular
# experiment date, in order to retain the genotype on that date.
minRootsPerGenotypePerPlateDate = 3

# Minimum number of measurements of a trait in a biorep of a particular genotype
# in order to calculate biorep mean for that genotype.
minRootsPerGenotypePerBioRep = 3

# P-value breakpoints for increasing numbers of stars.
pValueBreaks = c(0.001, 0.01, 0.05, 99)
theStars = c("***", "**", "*", "")
getStars = function(p) { p[is.na(p)] = 1; return(theStars[sapply(p, function(pi) match(TRUE, pi <= pValueBreaks))]) }
thePlusses = c("+++", "++", "+", "")
getPlusses = function(p) { p[is.na(p)] = 1; return(thePlusses[sapply(p, function(pi) match(TRUE, pi <= pValueBreaks))]) }

# Synthetic data columns can be created by concatenating data in other columns.
# Specify such columns here.  This is particularly useful for nested factors.
# If column B is nested within column A, then creating a new column A.B that
# joins the two column values (with a "." separator) allows specification of
# A.B within a model, resulting in a nested effect (lmer() manual recommends
# this approach).
# For each synthetic data column, specify a template string here, with data
# column names surrounded by % characters.  These will be replaced by the value
# in those columns to form a new string that is placed in a new column whose
# name is the name of the string in this vector. #####
synColTemplates = c(
    # (None)
    # date.gd="%plateDate%.%germDay%",
    NULL) # NULL is not included in a vector.

# Additional synthetic columns might be created by applying a mathematical
# relationship, represented by an R formula, to other columns.  Specify the
# formulas in this list, with the formula element names being the new column
# names.  Use I() to surround arithmetic expressions.
formulaCols = list(
    # (None)
    # pg.age="I(platingAge - germAge)"
    )

################################################################################
# Create log file.
################################################################################

# Create the output folder if it doesn't exist.
if (!file.exists(analysisDir))
    dir.create(analysisDir)

# Make an empty log file to start logging our activity into.
setDefaultLogFile(logFile)
printLog("RootRNAseqJune2013 phenotype modelling analysis log file", logFile, "\n", append=FALSE)

################################################################################
# Get traits that were measured.
################################################################################

# Get traits.
dfTraits = getTraits()

# Remove the traits we don't want.
dfTraits = dfTraits[!rownames(dfTraits) %in% rmvTraits,]

# Save the dfTraits data frame in the output folder.
write.table(dfTraits, traitsFile, sep="\t", row.names=TRUE, col.names=TRUE)
# dfTraits = read.table(traitsFile, sep="\t", header=TRUE, row.names=1, stringsAsFactors=FALSE)

# Get vector of traits and number of traits.
traits = rownames(dfTraits)
ntraits = length(traits)
printLog("There are", ntraits, "traits in total:\n", getPrint(dfTraits, "    "), "\n")

# Also get the trait groups and save them in a file.  We don't use them here.
dfGroups = getTraitGroups()
write.table(dfGroups, groupsFile, sep="\t", row.names=FALSE, col.names=TRUE)
rm(dfGroups)
# dfGroups = read.table(groupsFile, header=TRUE, sep="\t", stringsAsFactors=FALSE)

################################################################################
# Read the input data file.  This file has already been thoroughly vetted for
# correctness by an earlier pre-processing stage.  It should never be edited.
# If something needs changed in it, change the raw input data or change the
# pre-processing code that makes it.
# If columns are changed, colClasses will need to be changed too.
################################################################################

dfRoots = read.xlsx2(processedDataPath, sheetIndex=1, header=TRUE,
    stringsAsFactors=FALSE, colClasses=processedDataColClasses)

################################################################################
# Write the data frame to the output folder and show a summary.
################################################################################

# Write the data frame to the output folder.
make.xlsx(dfRoots, copyOfInputDataFile, freezeTopRow=TRUE, halignDefault="CENTER")

# Show summary of data.
printLog("Data column names:\n")
printLog(colnames(dfRoots), "\n")
printLog("Data summary:\n")
printLog(getPrint(summary(dfRoots)))

################################################################################
# Get genotypes and bioreps and plateDates.
################################################################################

# Get genotypes and number of genotypes.
genotypes = unique(dfRoots$genotype)
ngenotypes = length(genotypes) #how long is the vector (140 #genotypes)
printLog("There are", ngenotypes, "genotypes in total:\n")
printLog(genotypes, "\n", fill=70, labels="   ")

# Get bioreps and number of bioreps.
bioreps = unique(dfRoots$replicate)
nbioreps = length(bioreps)
printLog("There are", nbioreps, "bioreps in total: ", paste(bioreps, collapse=","), "\n")

# Get plateDate names and number of plateDates.
plateDates = unique(dfRoots$plateDate)
nplateDates = length(plateDates)
printLog("There are", nplateDates, "plateDates in total: ", paste(plateDates, collapse=","), "\n")

################################################################################
# Create synthetic data columns using the defined template strings and formulas.
################################################################################

for (col in names(synColTemplates))
    dfRoots[,col] = replaceTemplateString(synColTemplates[col], dfRoots)

for (col in names(formulaCols))
    dfRoots[,col] = eval(parse(text=formulaCols[[col]]), dfRoots)

################################################################################
# The data was obtained in multiple growth experiments, as indicated by the date
# in the plateDate column.  Samples from different experiments were divvied up
# into four biological replicates numbered REP2 through REP5 (REP1 was a
# practice rep).  Each biorep might have samples from more than one growth
# experiment.  The last one centimeter of roots that were 3 or 4 days post
# germination was harvested and RNA extracted for an RNAseq experiment.  Just
# prior to harvest, the plate was imaged.  Each root that grew, didn't hit plate
# walls, and wasn't obscured by labels was measured by tracing it and recording
# the centerline coordinates.  These coordinates were analyzed to generate the
# traits given in dfTraits.
# There are no NA entries in the trait data, all roots in dfRoots have a value
# for each trait.
# In some cases, some ILs may have insufficient roots to be analyzed.  While the
# insufficient data could be passed to the modelling code, we prefer to remove
# those ILs here and list them up front so we know they have been excluded.
# Also, we remove roots that are not either 3 or 4 days post germination.
################################################################################

# Make reduced data set from the data frame df.
# Arguments:
#   minPts: remove genotypes for which there are fewer than minPts roots in each
#       plateDate, and at least one plateDate with >= minPts roots.  Any data
#       points in a plateDate with fewer than "minPts" data points are discarded.
#       Any genotypes without at least one plateDate remaining are discarded.
#   rmvCollide: if TRUE, remove roots whose "collide" column is TRUE (root
#       collided with another).
#   rmvAlong: if TRUE, remove roots whose "along" column is TRUE (root grew
#       alongside another).
#   rmvRootsLessThanDAG: remove roots less than this many days old post-germination.
#   rmvRootsMoreThanDAG: remove roots more than this many days old post-germination.
# Return NULL if all data is removed, else return a data frame containing the
#       remaining data, with several columns converted to factors.
reduceData = function(df, minPts, rmvCollide, rmvAlong, rmvRootsLessThanDAG, rmvRootsMoreThanDAG, verbose=TRUE)
    {
    # Remove roots with collide=TRUE.
    if (rmvCollide)
        {
        N = nrow(df)
        Nc = sum(df$collide)
        Nr = N-Nc
        if (Nc > 0)
            {
            if (verbose)
                printLog("Out of", N, "total roots,", Nc, "were removed because they collide with another one, which leaves", Nr, "roots\n")
            df = df[!df$collide,]
            }
        }

    # Remove roots with along=TRUE.
    if (rmvAlong)
        {
        N = nrow(df)
        Na = sum(df$along)
        Nr = N-Na
        if (Na > 0)
            {
            if (verbose)
                printLog("Out of", N, "total roots,", Na, "were removed because they grew alongside another one, which leaves", Nr, "roots\n")
            df = df[!df$along,]
            }
        }

    # Remove roots outside of required germination age range.
    N = nrow(df)
    wrongAgeRange = (is.na(df$rootAge) | df$rootAge < rmvRootsLessThanDAG | df$rootAge > rmvRootsMoreThanDAG)
    Nw = sum(wrongAgeRange)
    Nr = N-Nw
    if (Nw > 0)
        {
        if (verbose)
            printLog("Out of", N, "total roots,", Nw, "were removed because they were not between",
                rmvRootsLessThanDAG, "and", rmvRootsMoreThanDAG, "days after\n",
                "germination when harvested (or had unknown age), which leaves", Nr, "roots\n")
        df = df[!wrongAgeRange,]
        }

    # Save original genotypes and plateDates.
    old.genotypes = unique(df$genotype)
    old.ngenotypes = length(old.genotypes)
    old.plateDates = unique(df$plateDate)
    old.nplateDates = length(old.plateDates)

    # Count number of roots of each genotype in each plateDate.
    numRoots = tapply(1:nrow(df), list(df$genotype, df$plateDate), length)
    numRoots[is.na(numRoots)] = 0
    # Remove rows not meeting the minPts requirement.
    df = df[apply(df, 1, function(d) numRoots[d["genotype"],d["plateDate"]] >= minPts),]
    # Check for no data left.
    if (nrow(df) == 0)
        {
        if (verbose)
            printLog("No samples that satisfy the minimal data criteria!!!\n")
        return(NULL)
        }

    # Convert columns likely to be used in linear modelling to factors.
    df$replicate = factor(df$replicate)
    df$genotype = factor(df$genotype)
    df$platePos = factor(df$platePos)
    df$plateLetter = factor(df$plateLetter)
    df$germDay = factor(as.character(df$germDay))
    df$collide = factor(df$collide)
    df$plateDate = factor(df$plateDate)
    # Convert all synthetic columns too.
    for (col in names(synColTemplates))
        df[,col] = factor(df[,col])

    # Set m82 as reference level.
    if (!any(df$genotype == "M82"))
        {
        if (verbose)
            printLog("No M82 samples that satisfy the minimal data criteria!!!\n")
        return(NULL)
        }
    df$genotype = relevel(df$genotype, ref="M82")

    # If verbose, display number of roots over all genotypes.
    if (verbose)
        {
        numRootsInGenotype = tapply(1:nrow(df), df$genotype, length)
        numRootsInGenotype = numRootsInGenotype[!(names(numRootsInGenotype) %in% parents)]
        printLog("   Minimum number of roots for any IL:", min(numRootsInGenotype), "\n")
        printLog("   Maximum number of roots for any IL:", max(numRootsInGenotype), "\n")
        printLog("   Mean number of roots for all ILs:", mean(numRootsInGenotype), "\n")
        printLog("   SD for number of roots for all ILs:", sd(numRootsInGenotype), "\n")
        printLog("\n")
        }

    # Count number of roots remaining for each genotype and plateDate.
    numRoots = tapply(1:nrow(df), list(df$genotype, df$plateDate), length)
    numRoots[is.na(numRoots)] = 0

    # If verbose, display number of roots over all plateDates.
    if (verbose)
        {
        numRoots = numRoots[!rownames(numRoots) %in% parents,]
        printLog("   Min and max number of roots for ILs in each plateDate:\n")
        for (plateDate in colnames(numRoots))
            {
            x = numRoots[, plateDate]
            x = x[x != 0]
            printLog("    ", format(plateDate, width=6), " min:", format(min(x), width=3), " max:", format(max(x), width=3), "\n")
            }
        printLog("\n")
        }

    # Get remaining genotypes and plateDates.
    new.genotypes = levels(df$genotype)
    new.ngenotypes = length(new.genotypes)
    new.plateDates = levels(df$plateDate)
    new.nplateDates = length(new.plateDates)

    # Tell the user which genotypes were eliminated and which retained as a result of above.
    rmvd.genotypes = setdiff(old.genotypes, new.genotypes)
    rmvd.ngenotypes = length(rmvd.genotypes)
    printLog("   There are", new.ngenotypes, "genotypes remaining after removing those with insufficient data:\n")
    printLog(new.genotypes, "\n", fill=70, labels="   ")
    printLog("   There were", rmvd.ngenotypes, "genotypes removed.  They were:\n")
    printLog(rmvd.genotypes, "\n", fill=70, labels="   ")

    # Return the reduced data frame.
    return(df)
    }

# Now reduce the overall data set, removing ILs with insufficient data.
printLog("Reducing data set to include only those genotypes with at least",
    minRootsPerGenotypePerPlateDate, "points in each plateDate.\n")
data.all = reduceData(dfRoots, minPts=minRootsPerGenotypePerPlateDate, rmvCollide=rmvCollideRoots,
    rmvAlong=rmvAlongRoots, rmvRootsLessThanDAG=rmvRootsLessThanDAG, rmvRootsMoreThanDAG=rmvRootsMoreThanDAG)

# Write the reduced data frame to a file.
write.table(data.all, vettedInputDataFile, sep="\t", row.names=FALSE)

################################################################################
# Compute means of each trait in each biorep of each genotype.  Put results in a
# data frame with these columns: "genotype", "biorep", and the trait names.
################################################################################

printLog("Calculating trait means in each biorep of genotype, requiring ",
    minRootsPerGenotypePerBioRep, "roots per biorep\n", sep="")
dfBiorepMeans = NULL
dfLostBioreps = NULL
for (gt in genotypes)
    {
    for (biorep in bioreps)
        {
        df = data.all[data.all$genotype == gt & data.all$replicate == biorep,]
        t.means = sapply(traits, function(t)
            {
            V = df[!is.na(df[,t]), t]
            if (length(V) < minRootsPerGenotypePerBioRep)
                return(NA)
            return(mean(V))
            })
        lostTraits = traits[is.na(t.means)]
        if (length(lostTraits) > 0)
            dfLostBioreps = rbind(dfLostBioreps, data.frame(genotype=gt, biorep=biorep,
                traits=paste(lostTraits, collapse=","), stringsAsFactors=FALSE))
        dfBiorepMeans = rbind(dfBiorepMeans, data.frame(genotype=gt, biorep=biorep,
            t(t.means), stringsAsFactors=FALSE))
        }
    }
if (!is.null(dfLostBioreps))
    {
    ord = order.special(biorep=dfLostBioreps$biorep, genotype=dfLostBioreps$genotype,
        lead0="genotype", first=c("M82", "PENN"))
    dfLostBioreps = dfLostBioreps[ord,]
    rownames(dfLostBioreps) = NULL
    printLog("Missing some biorep means for some traits in some genotypes:\n",
        getPrint(dfLostBioreps, "   "), "\n", sep="")
    }

# Write the means data frame to a file.
# col.names=NA will leave a blank cell in the name of the Genotypes column
write.table(dfBiorepMeans, biorepMeansFile, sep="\t", row.names=FALSE)

################################################################################
# Final model fit using chosen models.
################################################################################

# The models chosen through model testing are:
#   trait       model formula (in lmer terms; lme uses list(1|germAge, 1|platingAge) for random effects)
#   ----------- -------------------------------------------------------------------------------------
#   straightLen genotype + collide + (1|germAge) + (1|platingAge) + (1|plateDate)
#   rootLen     genotype + collide + (1|germAge) + (1|platingAge) + (1|plateDate)
#   tipAng      genotype + collide + (1|germAge) + (1|plateDate)
#   avgAbsAng   genotype + collide + (1|germAge) + (1|plateDate)
{
if (analysisVariant == "Orig")
    {
#   wholeAng    genotype + collide + (1|germAge) + (1|plateDate)
#   quarterAng  genotype + collide + (1|germAge) + (1|plateDate)
#   tenthAng    genotype + collide + (1|germAge) + (1|plateDate)
    }
else if (analysisVariant == "thirdsAngs" || analysisVariant == "thirdsAngB")
    {
#   first3rdAng genotype + collide + (1|germAge) + (1|plateDate)
#   mid3rdAng   genotype + collide + (1|germAge) + (1|plateDate)
#   last3rdAng  genotype + collide + (1|germAge) + (1|plateDate)
    }
}

# We will use lme(), not lmer(), to run the model, because lme() makes it easy to
# get p-values for each coefficient.  Make a list with one element per trait,
# the element being a sublist whose name is the trait name, whose first element
# is a formula for the fixed effects, and whose remaining elements are random
# effects formulas, one element per random effect, as strings.
{
if (analysisVariant == "Orig")
    {
    forms = list(
        straightLen=list("straightLen ~ 1 + genotype + collide", "~1|germAge", "~1|platingAge", "~1|plateDate"),
        rootLen=list("rootLen ~ 1 + genotype + collide", "~1|germAge", "~1|platingAge", "~1|plateDate"),
        tipAng=list("tipAng ~ 1 + genotype + collide", "~1|germAge", "~1|plateDate"),
        avgAbsAng=list("avgAbsAng ~ 1 + genotype + collide", "~1|germAge", "~1|plateDate"),
        wholeAng=list("wholeAng ~ 1 + genotype + collide", "~1|germAge", "~1|plateDate"),
        quarterAng=list("quarterAng ~ 1 + genotype + collide", "~1|germAge", "~1|plateDate"),
        tenthAng=list("tenthAng ~ 1 + genotype + collide", "~1|germAge", "~1|plateDate"))
    }
else if (analysisVariant == "thirdsAngs" || analysisVariant == "thirdsAngB")
    {
    forms = list(
        straightLen=list("straightLen ~ 1 + genotype + collide", "~1|germAge", "~1|platingAge", "~1|plateDate"),
        rootLen=list("rootLen ~ 1 + genotype + collide", "~1|germAge", "~1|platingAge", "~1|plateDate"),
        tipAng=list("tipAng ~ 1 + genotype + collide", "~1|germAge", "~1|plateDate"),
        avgAbsAng=list("avgAbsAng ~ 1 + genotype + collide", "~1|germAge", "~1|plateDate"),
        first3rdAng=list("first3rdAng ~ 1 + genotype + collide", "~1|germAge", "~1|plateDate"),
        mid3rdAng=list("mid3rdAng ~ 1 + genotype + collide", "~1|germAge", "~1|plateDate"),
        last3rdAng=list("last3rdAng ~ 1 + genotype + collide", "~1|germAge", "~1|plateDate"))
    }
else
    stop("Bad analysisVariant value")
}

##################################
# Make results matrices.
# We will write data from these matrices to files after we compute them.
model_matrix = matrix(nrow=ngenotypes, ncol=ntraits, dimnames=list(genotypes, traits))
#model_matrix

r.N = model_matrix
r.mean = model_matrix
r.sd = model_matrix
r.cv = model_matrix
r.sem = model_matrix
r.pvals = model_matrix
r.qvals = model_matrix
r.stars = model_matrix
r.plus = model_matrix

##################################
# Run loop for each trait.  Results will go into the above matrices.
for (t in traits)
    {
    printLog("Processing trait: ", t, "\n")

    # Get the reduced data frame derived above.
    data.r = data.all

    # Convert the formulas from strings to actual R formula data types.
    L = forms[[t]]
    fixed.formula = as.formula(L[[1]])
    random.formulas = sapply(L[-1], as.formula)

    # Fit the model.
    lme1 = lme(fixed.formula, random=random.formulas, data=data.r, na.action=na.omit)

    # Get a summary.
    lme1.summary = summary(lme1)

    # Retrieve p-values.
    pvals = lme1.summary$tTable[,"p-value"]
    pvals = pvals[grepl("^genotype", names(pvals))] # Get only those p-values for the genotype coefficients.
    names(pvals) = sub("genotype", "", names(pvals)) # Strip Genotype from names so they will match.
    r.pvals[names(pvals), t] = pvals # Aligns things correctly.

    # Make a plus-sign representation of the p-values.
    r.plus[names(pvals), t] = getPlusses(pvals)

    # Perform multiple hypothesis testing correction to generate corrected
    # p-values = q-values.  This produces warnings of unknown cause.  Use
    # p.adjust instead, not sure of original reason for using a Dunnett test.
    #lme1.glht = glht(lme1, linfct=mcp(genotype="Dunnett"))
    # Summary takes several minutes and produces warnings.
    #lme1.glht.summary = summary(lme1.glht)
    # Retrieve corrected p-values = q-values.
    #qvals = lme1.glht.summary$test$pvalues
    #names(qvals) = names(lme1.glht.summary$test$coefficients)
    #names(qvals) = sub(" - M82", "", names(qvals), fixed=TRUE)
    #r.qvals[names(qvals), t] = qvals

    # Perform multiple hypothesis testing correction to generate corrected
    # p-values = q-values, using B-H aka FDR method.
    qvals = p.adjust(pvals, method="fdr")
    r.qvals[names(qvals), t] = qvals

    # Retrieve corrected p-values = q-values.
    #qvals = lme1.glht.summary$test$pvalues
    #names(qvals) = names(lme1.glht.summary$test$coefficients)
    #names(qvals) = sub(" - M82", "", names(qvals), fixed=TRUE)

    # Make a stars representation of the corrected p-values.
    r.stars[names(qvals), t] = getStars(qvals)

    # Calculate number of replicates for each genotype.
    numByGenotype = tapply(data.r[,t], data.r$genotype, function(V) sum(!is.na(V)))
    r.N[names(numByGenotype), t] = numByGenotype # Align things correctly

    # Calculate means.
    meansByGenotype = tapply(data.r[,t], data.r$genotype, mean, na.rm=T)
    r.mean[names(meansByGenotype), t] = meansByGenotype # Align things correctly

    # Calculate standard deviations.
    sdsByGenotype = tapply(data.r[,t], data.r$genotype, sd, na.rm=T)
    r.sd[names(sdsByGenotype), t] = sdsByGenotype # Align things correctly

    # Calculate coefficients of variation.
    CVsByGenotype = tapply(data.r[,t], data.r$genotype, function(x) sd(x, na.rm=T)/mean(x, na.rm=T))
    r.cv[names(CVsByGenotype), t] = CVsByGenotype # Align things correctly

    # Calculate standard errors.
    semsByGenotype = tapply(data.r[,t], data.r$genotype, function(x) (sd(x, na.rm=T)/sqrt(length(x[!is.na(x)]))))
    r.sem[names(semsByGenotype), t] = semsByGenotype # Align things correctly

    # The glht summary produced sigma values that were previously being set aside
    # as standard error values, but never used.  What are these?
    #lme1.glht.summary.stderr = lme1.glht.summary$test$sigma
    #names(lme1.glht.summary.stderr) = sub(" - M82", "", names(lme1.glht.summary.stderr), fixed=TRUE)
    }
#r.N
#r.mean
#r.sd
#r.cv
#r.sem
#r.pvals
#r.qvals
#r.stars
#r.plus

################################################################################
# Write out useful tables of results.
################################################################################

# Write matrices of information about the model fits to files.
# col.names=NA will leave a blank cell in the name of the Genotypes column
write.table(r.N, getPerGenotypeDataFilename("N.tsv"), sep="\t", col.names=NA)
write.table(r.mean, getPerGenotypeDataFilename("mean.tsv"), sep="\t", col.names=NA)
write.table(r.sd, getPerGenotypeDataFilename("sd.tsv"), sep="\t", col.names=NA)
write.table(r.cv, getPerGenotypeDataFilename("cv.tsv"), sep="\t", col.names=NA)
write.table(r.sem, getPerGenotypeDataFilename("sem.tsv"), sep="\t", col.names=NA)
write.table(r.pvals, getPerGenotypeDataFilename("p-value.tsv"), sep="\t", col.names=NA)
write.table(r.plus, getPerGenotypeDataFilename("plus.tsv"), sep="\t", col.names=NA) # NA and "" are different, NA means not enough data, and M82=NA
write.table(r.qvals, getPerGenotypeDataFilename("q-value.tsv"), sep="\t", col.names=NA)
write.table(r.stars, getPerGenotypeDataFilename("stars.tsv"), sep="\t", col.names=NA) # NA and "" are different, NA means not enough data, and M82=NA

# Create and write out a table of IL data, all traits.
printLog("Making file (_summaryData.tsv) of all of the above info for all traits, as a data frame.\n")
df = NULL
for (t in traits)
    {
    dfx = data.frame(trait=t, genotype=rownames(r.N), N=r.N[,t], mean=r.mean[,t], sd=r.sd[,t], cv=r.cv[,t],
        sem=r.sem[,t], pvalue=r.pvals[,t], qvalue=r.qvals[,t], stars=r.stars[,t], plus=r.plus[,t],
        stringsAsFactors=FALSE)
    df = rbind(df, dfx)
    }
write.table(df, getPerGenotypeDataFilename("summaryData.tsv"), sep="\t", row.names=FALSE)

# Create and write out a table of ILs significant in any trait.

printLog("Making file (_sigGenotypes.tsv) containing list of all qvalue-significant genotypes for each trait.\n")
df = NULL
for (t in traits)
    {
    sigIdxs = which(nchar(r.stars[,t]) > 0)
    dfx = data.frame(trait=t, genotype=rownames(r.N)[sigIdxs], N=r.N[sigIdxs,t],
        mean=r.mean[sigIdxs,t], sd=r.sd[sigIdxs,t], cv=r.cv[sigIdxs,t],
        sem=r.sem[sigIdxs,t], pvalue=r.pvals[sigIdxs,t], qvalue=r.qvals[sigIdxs,t],
        stars=r.stars[sigIdxs,t], plus=r.plus[sigIdxs,t], stringsAsFactors=FALSE)
    df = rbind(df, dfx)
    }
write.table(df, getPerGenotypeDataFilename("sigGenotypes.tsv"), sep="\t", row.names=FALSE)

################################################################################
# End.
################################################################################
