################################################################################
# This file contains R code to plot root phenotype model fitting results from
# the Brady Lab root RNAseq experiment using tomato introgression lines (ILs)
# with RNAseq data first obtained in June 2013.  This code reads the results of
# model fitting and makes various plots.
#
# EDIT FILE Include_setAnalysisAndDate.R BEFORE RUNNING THIS CODE.
################################################################################

# Set working directory.
setwd("~/Documents/UCDavis/BradyLab/GrowthExperiments/Tomato/RootRNAseqJun2013_Phenotypes")

################################################################################
# Include files.
################################################################################

source("Include_setAnalysisAndDate.R")
source("Include_general.R")
source("Include_xlsx.R")
source("Include_dataColumnTypes.R")
source("Include_Heatmaps.R")
source("Include_VennDiagrams.R")
phenoRNAseqRootSep2011Dir = "~/Documents/UCDavis/BradyLab/CellCountingGravity"
source(paste(phenoRNAseqRootSep2011Dir, "Include_phenoRNAseqRootSep2011.R", sep=PATHSEP))

################################################################################
# Load libraries used by this.
################################################################################

library("beanplot")
library("cluster")

################################################################################
# Set basic constants controlling the analysis.
################################################################################

# Significance threshold for Venn diagram.
venn.q.thresh = 0.05

# Use TRUE to generate images tailored for a paper, FALSE for ourselves.
paper = TRUE

# Set filename prefix, which prefixes both input and output files, and includes
# the input/output folder at the beginning of it.
rootOutputPrefix = analysisOutputPrefix
if (paper)
    rootOutputPrefix = paste(rootOutputPrefix, "paper_", sep="")

# Input data filenames.
traitsFile = paste(analysisOutputPrefix, "traits.tsv", sep="")
vettedInputDataFile = paste(analysisOutputPrefix, "inputDataFiltered.tsv", sep="")
modelSummaryDataFile = paste(analysisOutputPrefix, "summaryData.tsv", sep="")

# Use FALSE to suppress annotation of number of samples on bar graphs.
addNumSampleAnnotation = !paper

# Use TRUE to dim the names of non-sig. ILs on X-axis label of bar graphs.
dimInsignificantNames = TRUE

# Use TRUE to color parent labels with parent colors.
colorParentLabels = TRUE

# Use FALSE to suppress error bars.
addErrorBars = TRUE

# Use FALSE to suppress "+" signs above bars indicating significance of uncorrected p-values.
showPlusForUncorrectP = !paper

# Number of standard deviations beyond which to call a data point an outlier.
xsd = 3.0

# Use an empty list to not make any subset heatmaps, else each list element is a
# character vector of genotypes to include in the subset heatmap.
makeHeatmapSubsetGTs = list() # list(A=c("PENN", "IL2-3"), B=c("PENN", "IL7-5"))

# P-value breakpoints for increasing numbers of stars.
pValueBreaks = c(0.001, 0.01, 0.05, 99)
theStars = c("***", "**", "*", "")
getStars = function(p) { p[is.na(p)] = 1; return(theStars[sapply(p, function(pi) match(TRUE, pi <= pValueBreaks))]) }
thePlusses = c("+++", "++", "+", "")
getPlusses = function(p) { p[is.na(p)] = 1; return(thePlusses[sapply(p, function(pi) match(TRUE, pi <= pValueBreaks))]) }

# Bar graph bar colors.
sigBarColors = c(`***`="gray40", `**`="gray55", `*`="gray70") # Corresponding to all but last pValueBreaks entry.
colorTheseGenotypes = c() # Color these genotypes special colors.  Example: c("IL7-4-1", "IL7-5")
colorOfThoseGenotypes = c() # The colors to use.  Example: c("blue", "gold")
# Always include the parents in the special bar colors.
colorTheseGenotypes = c(colorTheseGenotypes, parents)
colorOfThoseGenotypes = c(colorOfThoseGenotypes, parentColorsLight)

# Set this TRUE to use colors to differentiate plateDates in points plotted on
# bean plots.
showDateInColor = FALSE

# Paths to Spalding rotation modelling results.
#spaldingDir = paste("~", "Documents", "UCDavis", "BradyLab", "AngleGeneAnalysis",
#    "SpaldingLab", "Analysis_Rot", ?????, sep=PATHSEP)
#allTipAnglesSummaryFile = paste(spaldingDir, paste(fileDate, "_allILtipangles_min4_summaryData.tsv", sep=""), sep=PATHSEP)
#allILderivSummaryFile = paste(spaldingDir, paste(fileDate, "_allILderiv_min4_summaryData.tsv", sep=""), sep=PATHSEP)

################################################################################
# Get traits that were measured.
################################################################################

# Get traits that were analyzed.
dfTraits = read.table(traitsFile, sep="\t", header=TRUE, row.names=1, stringsAsFactors=FALSE)
traits = rownames(dfTraits)

################################################################################
# Read the filtered input data file.  This file has already been thoroughly
# vetted for correctness by an earlier pre-processing stage, and was also
# filtered by the modelling code to remove data that was insufficient for
# modelling.
################################################################################

data.all = read.table(vettedInputDataFile, sep="\t", header=TRUE, row.names=NULL, stringsAsFactors=FALSE)

################################################################################
# Read the model output giving significance of different traits in different ILs.
################################################################################

dfModelResults = read.table(modelSummaryDataFile, sep="\t", header=TRUE, row.names=NULL, stringsAsFactors=FALSE)

################################################################################
# Get genotypes.
################################################################################

# Get genotypes and number of genotypes.
genotypes = unique(data.all$genotype)
ngenotypes = length(genotypes)
cat("There are", ngenotypes, "genotypes in total:\n")
cat(genotypes, "\n", fill=70, labels="   ")

################################################################################
# Do a correlation matrix between traits.
################################################################################

gts = setdiff(genotypes, "PENN")
mtx = matrix(NA, nrow=length(gts), ncol=length(traits), dimnames=list(gts, traits))
df = dfModelResults[dfModelResults$genotype %in% gts, c("trait", "genotype", "mean")]
for (i in 1:nrow(df))
    mtx[df$genotype[i], df$trait[i]] = df$mean[i]
R = cor(mtx, use="complete.obs")
R = signif(R, 2)
R

################################################################################
# Venn diagrams.
################################################################################

# Make Venn diagrams in this pdf file.
pdf(paste(rootOutputPrefix, "vennDiagrams.pdf", sep=""), height=10, width=10)

# Make a venn diagram showing the number of significant ILs for each of the
# angle traits, and how they intersect.
venn.traits = traits[grepl("Ang", traits)]
#venn.traits
venn.whichSig = sapply(venn.traits, function(t)
    {
    r = dfModelResults$genotype[dfModelResults$trait == t & dfModelResults$qvalue <= venn.q.thresh]
    r = r[!is.na(r)]
    r = r[r %in% genotypes]
    r = r[r != "PENN"] # Remove PENN
    r = sub("IL", "", r) # Remove "IL" so more space.
    return(r)
    }, simplify=FALSE)
plotVenn5(venn.whichSig, main="Intersections of ILs Sig. for Angle in each of 5 New Angle Measures",
    showSetSize=TRUE, elementsIfLt=20, cex.labels=2.2, cexSizeMult=1.2, cexShrinkPerElement=0.97,
    cexMin=1.6, cexMax=2, kShowPt=2,
    nudgeIntersectionLabels=list(BC=c(-5,5), BCDE=c(13,-8), ABCDE=c(-20,5), ABE=c(0,6), ABDE=c(25,10),
        BDE=c(-11,7), DE=c(0,-10), AE=c(-5,2), BCD=c(-15,5)))
text(50, 350, paste("(q <= ", signif(venn.q.thresh, 2), ")", sep=""), cex=1.5, font=3)

# Similar to above, but lacking a 6-way Venn, replace tipAng with gravity_d5 from
# earlier experiment.
# Read number of stars associated with adjusted p-values of comparison of
# genotype to M82 from earlier experiment and extract gravity_d5 sig. ILs.
traitNstars = read.table(traitStarsFile, header=TRUE, stringsAsFactors=FALSE)
traitNstars[is.na(traitNstars)] = 0
venn.nstars.thresh = nchar(theStars[pValueBreaks == venn.q.thresh])
gravity_d5.sig = rownames(traitNstars)[traitNstars$gravity_d5 >= venn.q.thresh]
gravity_d5.sig = gravity_d5.sig[gravity_d5.sig != "PENN"] # Remove PENN
gravity_d5.sig = sub("IL", "", gravity_d5.sig) # Remove "IL" so more space.
names(venn.whichSig)[names(venn.whichSig) == "tipAng"] = "gravity_d5"
venn.whichSig$gravity_d5 = gravity_d5.sig
plotVenn5(venn.whichSig, main="Intersections of ILs Sig. for Angle in gravity_d5 plus 4 New Angle Measures",
    showSetSize=TRUE, elementsIfLt=20, cex.labels=2.2, cexSizeMult=1.2, cexShrinkPerElement=0.97,
    cexMin=1.6, cexMax=2, kShowPt=2,
    nudgeIntersectionLabels=list(BCDE=c(5,-5)))
text(50, 350, paste("(q <= ", signif(venn.q.thresh, 2), ")", sep=""), cex=1.5, font=3)

# Close pdf file.
dev.off()

# Read model results from Edgar Spalding rotation experiment, compute significant
# IL intersections with this experiment, and plot a Venn diagram.
# Comment this out.  We have two versions of the Spalding data, one with two sets
# of traits (tip angles and derivatives), and the other with six (those plus AWAY
# and THRU of each).
#dfTipAngles = read.table(allTipAnglesSummaryFile, header=TRUE, sep="\t", stringsAsFactors=FALSE)
#dfILderiv = read.table(allILderivSummaryFile, header=TRUE, sep="\t", stringsAsFactors=FALSE)
#sigILsTipAngles = unique(dfTipAngles$genotype[dfTipAngles$stars != ""])
#sigILsILderiv = unique(dfILderiv$genotype[dfILderiv$stars != ""])
#sigILsRootRNAseq = unique(unlist(venn.whichSig))
#venn.whichSig = list(FiveAngles=sigILsRootRNAseq, RotationTipAng=sigILsTipAngles, RotationTipRate=sigILsILderiv)

# Draw Venn diagram to TIFF file.
#vennFile = paste(rootOutputPrefix, "vennDiagram2.tiff", sep="")

#tedVenn(venn.whichSig, filename=vennFile,
#    cat.pos=c(0, 0, 155), cat.dist=c(0.025, 0.025, 0.0125), cat.cex=c(0.9, 0.9, 0.9),
#    main="Intersections of ILs Sig. for Angle in RNA-seq and Rotation Experiments")

################################################################################
# Make some plots to allow us to look for bad data.
################################################################################

pdf(paste(rootOutputPrefix, "dataplots.pdf", sep=""), height=10, width=10)
for (t in traits)
	{
	cat("Data plot for trait", t, "\n")

    # Get filtered data frame and change name of the current trait column to "trait"
    data.r = data.all[, c("genotype", t)]
    colnames(data.r)[2] = "trait"

	# X will be the genotype, Y will be the data value or mean.  Shape of the point will
	# indicate mean vs data point, round for means, triangle for data points.  Use black
	# for IL mean.  Plot only those data points that are more than 2 sd's away from IL mean.
	xlim = c(1, ngenotypes)
	ylim = range(pretty(c(0, 1.2*range(data.r$trait))))
	xmax = xlim[2]
	ymax = ylim[2]
	ydel = 0.03*ymax
	ypos = ymax
	meanPerIL = tapply(data.r$trait, data.r$genotype, mean)
	# Report X values by meanPerIL, apply this sorting order to remaining points() calls.
	xposIL = order(meanPerIL)
	names(xposIL) = genotypes[xposIL]
	plot(NA, type="n", xlim=xlim, ylim=ylim, xaxt="n",
		main=paste("Data for trait", t),
		xlab="", ylab=dfTraits[t, "units"])
	axis(1, at=1:length(xposIL), labels=names(xposIL), font=2, las=3, cex.axis=0.8) 
	pchDot = 20
	points(1:length(xposIL), meanPerIL[names(xposIL)], pch=pchDot, col="black")
	lines(1:length(xposIL), meanPerIL[names(xposIL)], col="black")
	legend(1, ypos, "IL Means", bty="n", pch=pchDot, col="black")
	ypos = ypos - ydel
    meanPerIL = tapply(data.r$trait, data.r$genotype, mean)
    points(1:length(xposIL), meanPerIL[names(xposIL)], pch=pchDot)
    lines(1:length(xposIL), meanPerIL[names(xposIL)])
    ypos = ypos - ydel
    sdPerIL = tapply(data.r$trait, data.r$genotype, sd)
    moreThanXSD = (abs(data.r$trait - meanPerIL[data.r$genotype]) > xsd*sdPerIL[data.r$genotype])
    x = data.r$genotype[moreThanXSD]
    y = data.r$trait[moreThanXSD]
    pchTriangle = 17
    points(xposIL[x], y, pch=pchTriangle)
    legend(1, ypos, paste("data pt >", xsd, "SDs"), bty="n", pch=pchTriangle)
    ypos = ypos - ydel
	}
dev.off()

################################################################################
# Bar plots.
################################################################################

# Loop for each trait and make a bar plot for each one.
pdf(paste(rootOutputPrefix, "barPlots.pdf", sep=""), width=11)
for (t in traits)
    {
    cat("Bar chart for trait", t, "\n")

    # Get bar and star data and sort it.  Remove NaN values.
    dfResults = dfModelResults[dfModelResults$trait == t,]
    rownames(dfResults) = dfResults$genotype
    dfResults = dfResults[genotypes,]
    mean_values = dfResults$mean
    names(mean_values) = dfResults$genotype
    err_values = dfResults$sem
    names(err_values) = dfResults$genotype
    bar_desc = "Bars = std. err. mean of all samples in genotype"
    stars = dfResults$stars
    names(stars) = dfResults$genotype
    stars[is.na(stars)] = ""
    plus = dfResults$plus
    names(plus) = dfResults$genotype
    plus[is.na(plus)] = ""

    # Sort by mean value of genotype.
    ord = order(mean_values)
    mean_values = mean_values[ord]
    err_values = err_values[ord]
    stars = stars[ord]
    plus = plus[ord]

    # Set bar colors.
    barcolors = rep("gray90", length(mean_values))
    names(barcolors) = names(mean_values)
    barcolors[names(stars)[stars != ""]] = sigBarColors[stars[stars != ""]]
    if (length(colorTheseGenotypes) > 0)
        barcolors[colorTheseGenotypes] = colorOfThoseGenotypes

    # Optionally use dimmer colors for x-axis labels of ILs that aren't significant,
    # except M82 and PENN are always shown in normal or parent color.
    barnames = names(mean_values)
    barLabelColors = rep("black", length(barnames))
    names(barLabelColors) = barnames
    if (colorParentLabels)
        barLabelColors[parents] = parentColors
    if (dimInsignificantNames)
        {
        insig = stars[barnames] == "" & barnames != "M82" & barnames != "PENN"
        barLabelColors[insig] = addAlpha(barLabelColors[insig], 0.25)
        }

    # Make some text size magnification estimates.
    cex = 50/length(mean_values) # A weird guess...
    dx = 0
    # Here's a correction for when there are very few bars.
    if (cex > 1)
        {
        cex = 1
        dx = -4/length(mean_values)
        }

    # Make a bar plot.
    ylim = range(pretty(c(0, 1.2*max(mean_values))))
    ymax = ylim[2]
    ydel = 0.04*ymax
    ypos = ymax
    yvals = mean_values
    xvals = 1:length(yvals)
    barplot(yvals, axisnames=FALSE, col=barcolors, space=0, ylim=ylim, font.axis=2,
        main=paste("\n\n\n\n", t, ": ", dfTraits[t, "desc"], sep=""), sub=ifelse(paper, "", fileDate),
        xlab="", ylab=paste(dfTraits[t, "desc"], " (", dfTraits[t, "units"], ")", sep=""))
    mtext(barnames, 1, line=0.5, at=xvals-0.5, col=barLabelColors, cex=cex, font=2, las=3)

    # Add error bars.
    if (addErrorBars)
        {
        arrows(xvals-0.5, yvals+err_values, xvals-0.5, yvals-err_values, angle=90, code=3, length=0.025)
        if (!paper)
            text(length(xvals)/3, ymax-3*ydel, bar_desc, cex=0.8, col="black", pos=4)
        }

    # Add stars above bars.
    for (x in xvals)
        {
        il = names(mean_values)[x]
        if (stars[il] != "")
            {
            y = mean_values[il] + 0.02*ymax
            color = sigBarColors[stars[il]]
            if (addErrorBars)
                y = y + err_values[il]
            text(x+dx, y, stars[il], cex=2*cex, font=2, srt=90, pos=4, offset=0, col=color)
            }
        }

    # Add plusses above bars for Siobhan results.
    if (showPlusForUncorrectP)
        {
        for (x in xvals)
            {
            il = names(mean_values)[x]
            if (plus[il] != "")
                {
                y = mean_values[il] + 0.1*ymax
                if (addErrorBars)
                    y = y + err_values[il]
                text(x-.5, y, plus[il], cex=cex, font=2, srt=90, pos=4, offset=0, col="blue")
                }
            }
        }

    # Add a legend for the stars and plusses.
    ypos = ypos - 2*ydel
    if (showPlusForUncorrectP)
        text(2, ypos, "raw", cex=1, pos=2)
    text(2, ypos, "FDR", cex=1, pos=4)
    text(length(xvals)/10, ypos, "threshold", cex=1, pos=4)
    for (i in 1:length(sigBarColors))
        {
        ypos = ypos - ydel
        if (showPlusForUncorrectP)
            text(2, ypos, thePlusses[i], cex=1, col=sigBarColors[i], pos=2)
        text(2, ypos-ydel/3, theStars[i], cex=1.5, col=sigBarColors[i], pos=4)
        text(length(xvals)/10, ypos, paste("p <", pValueBreaks[i]), cex=1, col=sigBarColors[i], pos=4)
        }

    # Add annotation of number of samples per plateDate to bars.
    if (addNumSampleAnnotation)
        {
        numRoots = tapply(1:nrow(data.all), list(data.all$genotype, data.all$plateDate), length)
        numRoots[is.na(numRoots)] = 0
        nPts = apply(numRoots, 1, paste, collapse=",")
        N = ncol(numRoots)
        for (i in (N-1):3)
            {
            S1 = paste("(^|,)", paste(rep("0", i), collapse=","), sep="")
            S2 = paste("\\10*", i, sep="")
            nPts = gsub(S1, S2, nPts)
            }
        text(xvals-0.5, ylim[1]+0.01*diff(ylim), nPts[names(mean_values)], cex=0.6, font=2, srt=90, pos=4, offset=0, col="black")
        }
    }
dev.off()

################################################################################
# Bean plots.
################################################################################

# Loop for each trait and make a bean plot for each one.
pdf(paste(rootOutputPrefix, "beanplots.pdf", sep=""), width=11)
for (t in traits)
    {
    cat("Bean plot for trait", t, "\n")

    # Get the list of genotypes for this trait, then sort data by mean value of genotype.
    dfResults = dfModelResults[dfModelResults$trait == t,]
    rownames(dfResults) = dfResults$genotype
    dfResults = dfResults[genotypes,]
    mean_values = dfResults$mean
    names(mean_values) = dfResults$genotype
    stars = dfResults$stars
    names(stars) = dfResults$genotype
    stars[is.na(stars)] = ""
    ord = order(mean_values)
    mean_values = mean_values[ord]
    stars = stars[ord]
    data.all$genotype = factor(data.all$genotype, levels=names(mean_values))

    # Set bean colors.
    beancolors = rep("gray90", length(mean_values))
    names(beancolors) = names(mean_values)
    beancolors[names(stars)[stars != ""]] = sigBarColors[stars[stars != ""]]
    if (length(colorTheseGenotypes) > 0)
        beancolors[colorTheseGenotypes] = colorOfThoseGenotypes
    beanCol = list()
    beanLineCol = "gold"
    sampLineCol = "black"
    beanAvgLineCol = "blue"
    for (col in beancolors)
        beanCol[[length(beanCol)+1]] = c(col, sampLineCol, beanLineCol, beanAvgLineCol)

    # Optionally use dimmer colors for x-axis labels of ILs that aren't significant,
    # except M82 and PENN are always shown in normal or parent color.
    beannames = names(mean_values)
    beanLabelColors = rep("black", length(beannames))
    names(beanLabelColors) = beannames
    if (colorParentLabels)
        beanLabelColors[parents] = parentColors
    if (dimInsignificantNames)
        {
        insig = stars[beannames] == "" & beannames != "M82" & beannames != "PENN"
        beanLabelColors[insig] = addAlpha(beanLabelColors[insig], 0.25)
        }

    # Make some text size magnification estimates.
    cex = 50/length(mean_values) # A weird guess...
    dx = 0.5
    # Here's a correction for when there are very few beans.
    if (cex > 1)
        {
        cex = 1
        dx = -4/length(mean_values)
        }

    # Make a bean plot.
    xmax = length(mean_values)
    xvals = 1:xmax
    the.formula = as.formula(paste(t, " ~ genotype", sep=""))
    beanplot(the.formula, data=data.all, show.names=FALSE, font.axis=2, grownage=8, log="",
        col=beanCol, border=beancolors, frame.plot=FALSE, beanlinewd=1, maxwidth=0.98,
        what=c(TRUE, TRUE, TRUE, FALSE), # Don't plot scatter plot, use different colors for plateDates.
        main=paste("\n\n\n\n", t, ": ", dfTraits[t, "desc"], sep=""), sub=ifelse(paper, "", fileDate),
        xlab="", ylab=paste(dfTraits[t, "desc"], " (", dfTraits[t, "units"], ")", sep=""))
    scatPt.lwd = 1
    scatPt.width = 0.25
    x1 = as.numeric(data.all$genotype)-scatPt.width/2
    x2 = x1+scatPt.width
    y = data.all[,t]
    cols = "black"
    if (showDateInColor)
        {
        dateColors = c(blue="1/17/13", red="1/29/13", green2="1/8/13", gold="11/20/12",
            brown="2/12/13", black="2/14/13", bisque3="2/19/13", darkslategray="2/21/13",
            darkslateblue="2/7/13", darkgoldenrod="3/1/13", chocolate3="6/28/13",
            burlywood4="7/2/13", blueviolet="7/9/13")
        cols = names(dateColors)[match(as.character(data.all$plateDate), dateColors)]
        }
    segments(x1, y, x2, y, lwd=scatPt.lwd, col=cols)
    mtext(beannames, 1, line=0.5, at=xvals, col=beanLabelColors, cex=cex, font=2, las=3)
    ymin = par("usr")[3]
    ymax = par("usr")[4]
    ydel = 0.04*ymax
    ypos = ymax

    # Add stars above beans.
    for (x in xvals)
        {
        il = beannames[x]
        if (stars[il] != "")
            {
            y = ymin # max(data.all[,t][data.all$genotype == il]) + 0.02*ymax
            color = sigBarColors[stars[il]]
            text(x+dx, y, stars[il], cex=2*cex, font=2, srt=90, pos=4, offset=0, col=color)
            }
        }

    # Add a legend for date colors.
    if (showDateInColor)
        legend("topright", legend=paste(dateColors, " (", names(dateColors), ")", sep=""),
            col=names(dateColors), lwd=scatPt.lwd, cex=0.6)

    # Add a legend for the stars and plusses (bean color).
    ypos = ypos - 2*ydel
    if (showPlusForUncorrectP)
        text(2, ypos, "raw", cex=1, pos=2)
    text(2, ypos, "FDR", cex=1, pos=4)
    text(length(xvals)/10, ypos, "threshold", cex=1, pos=4)
    for (i in 1:length(sigBarColors))
        {
        ypos = ypos - ydel
        if (showPlusForUncorrectP)
            text(2, ypos, thePlusses[i], cex=1, col=sigBarColors[i], pos=2)
        text(2, ypos-ydel/3, theStars[i], cex=1.5, col=sigBarColors[i], pos=4)
        text(length(xvals)/10, ypos, paste("p <", pValueBreaks[i]), cex=1, col=sigBarColors[i], pos=4)
        }
    }
dev.off()

################################################################################
# Dendrograms and heat map
################################################################################

cat("\nMaking heat map\n")

# Get a list of IL's that have at least one star in at least one trait.
sigILs = c()
for (t in traits)
    {
    dfResults = dfModelResults[dfModelResults$trait == t,]
    rownames(dfResults) = dfResults$genotype
    dfResults = dfResults[genotypes,]
    stars = dfResults$stars
    names(stars) = dfResults$genotype
    stars[is.na(stars)] = ""
    sigILs = union(sigILs, names(stars)[stars != ""])
    }
nonSigILs = setdiff(genotypes, sigILs)
cat("    significant ILs:", sigILs, "\n")
cat("    non-significant ILs:", nonSigILs, "\n")

# Create a data frame of number of stars for each IL/trait combination in sigILs and
# traits vectors.  For each one, test the trait's mean value in that IL with the mean
# value for M82, and use a NEGATIVE NUMBER OF STARS if the trait value is less than
# M82.  NA trait values are taken as NA.
dfStars = as.data.frame(matrix(NA, nrow=length(sigILs), ncol=length(traits),
    dimnames=list(sigILs, traits)))
for (t in traits)
    {
    dfResults = dfModelResults[dfModelResults$trait == t,]
    rownames(dfResults) = dfResults$genotype
    dfResults = dfResults[sigILs,]
    stars = dfResults$stars
    names(stars) = dfResults$genotype
    stars[is.na(stars)] = ""
    nstars = nchar(stars)
    means = dfResults$mean
    M82mean = dfModelResults$mean[dfModelResults$trait == t & dfModelResults$genotype == "M82"]
    lessThanM82 = (means < M82mean)
    nstars = nstars * ifelse(lessThanM82, -1, +1)
    dfStars[sigILs, t] = nstars
    }

# Sort data frame ILs in order.
dfStars = dfStars[sortILs(rownames(dfStars)),]

# Show IL summary.
sigILs = rownames(dfStars)
nonSigILs = setdiff(genotypes, sigILs)
cat("  Summary: ILs with stars:", sigILs, "\n")
cat("  Summary: ILs without stars:", nonSigILs, "\n")

# Write the data to a file.
write.table(dfStars, file=paste(rootOutputPrefix, "Heatmap.tsv", sep=""), sep="\t", row.names=TRUE)

# Make colors to use on the heatmap.  Make multiple copies of the heatmap, with different
# colors.
colorSets = list(
    list(ltM82color=parentColors["M82"], gtM82color="green4", NOTSIGcolor="black", NAcolor="gray",
         ltM82intensity=c(1,15,30), gtM82intensity=c(205,230,255),
         ltM82colorName=parentColorNames["M82"], gtM82colorName=parentColorNames["PENN"],
         subtitle="(red/green version for the non-color-impaired)",
         filenameSuffix="_NonColorImpaired"),
#    list(ltM82color="#D55E00", gtM82color="#009E73", NOTSIGcolor="black", NAcolor="#F0E442",
    list(ltM82color="#F56500", gtM82color="#00A275", NOTSIGcolor="black", NAcolor="#F0E442",
         ltM82intensity=c(1,25,50), gtM82intensity=c(190,222,255),
         ltM82colorName="red", gtM82colorName="blugrn",
         subtitle="(red/blue version for the color-impaired)",
         filenameSuffix="_ForColorImpaired1"),
#    list(ltM82color="#F0E442", gtM82color="#0072B2", NOTSIGcolor="black", NAcolor="#CC79A7", # "#009E73",
    list(ltM82color="#F0E442", gtM82color="#56B4E9", NOTSIGcolor="black", NAcolor="#E06200", # "#009E73",
         ltM82intensity=c(1,15,35), gtM82intensity=c(190,230,255),
         ltM82colorName="yellow", gtM82colorName="blue",
         subtitle="(yellow/blue version for the color-impaired)",
         filenameSuffix="_ForColorImpaired2")
    )

for (colorSet in colorSets)
    {
    cat("Making heatmap of significant ILs/traits:", colorSet$subtitle, "\n")

    # Put dendrograms and heatmap into a .pdf file.
    heatmapPdf = paste(rootOutputPrefix, "Heatmaps", colorSet$filenameSuffix, ".pdf", sep="")
    pdf(heatmapPdf, width=8, height=10)

    # Make colors to use on the heatmap.
    heatmapColors = makeHeatmapColors("#808080", colorSet$gtM82color, 127, colorSet$ltM82color, 127)
    xlabKey = paste("p-value threshold, ", colorSet$ltM82colorName, ": mean < M82 mean, ", colorSet$gtM82colorName, ": mean > M82 mean", sep="")
    nStarsBreaks = nchar(theStars)
    pvals = rep(NA, length(theStars))
    pvals[nStarsBreaks+1] = sub("0+$", "", as.character(pValueBreaks))
    pvals[1] = "1" # Change 99 to 1
    #pvals
    xvalKey.nstars = c(NA, 3, 2, 1, 0, 1, 2, 3)
    xvalKey = as.character(pvals[xvalKey.nstars+1])
    xvalKey[1] = "NA"
    #xvalKey
    # The first color is for value -4 = NA; following this are the values for
    # -3, -2, -1 stars (< M82), then 0 stars, then 1, 2, 3 stars (> M82).
    colors = c(colorSet$NAcolor, heatmapColors[colorSet$ltM82intensity],
        colorSet$NOTSIGcolor, heatmapColors[colorSet$gtM82intensity])   
    breaks = seq(-4, +3, length.out=length(colors)+1)

    # Make a heatmap with no dendrograms.
    makeHeatmap(dfStars, colDendro=NULL, rowDendro=NULL, xlabKey=xlabKey, xvalKey=xvalKey,
        margins=c(bottom=5,left=5,top=10,right=5),
        heatmapAreaSize=c(80,80), heatmapSize=c(15,83), keySize=c(60,35), keySpace=c(13,45), key=3,
        lftLabSize=c(35,0.5), topLabSize=c(15,0.5),
        colsep=2:ncol(dfStars), rowsep=2:nrow(dfStars), sepcolor="gray", sepwidth=c(0.025,0.05),
        density.info="none", cexRow=1, cexCol=1, cexLabKey=0.6, cex.main=1.2,
        valueForNA=-4, heatmapColors=colors, breaks=breaks,
        topLabCol=TRUE, btmLabCol=FALSE, lftLabRow=TRUE, rgtLabRow=FALSE, srtLabCol=90,
        titleRowDendrogram="ILs clustered by abs. Pearson corr. of each trait's p-value for comparison to M82",
        titleHeatmap="Heatmap: ILs with significant p-value in trait comparison to M82\nILs clustered by abs. Pearson corr. of p-value",
        subtitleHeatmap=colorSet$subtitle)

    # Make a heatmap with dendrograms.
    makeHeatmap(dfStars, colDendro=NULL, rowDendro=NA, xlabKey=xlabKey, xvalKey=xvalKey,
        margins=c(bottom=5,left=30,top=10,right=25),
        heatmapAreaSize=c(40,70), heatmapSize=c(30,85), keySize=c(100,25), keySpace=c(0,30), key=3,
        lftLabSize=c(15,0.5), rgtLabSize=c(0,0), btmLabSize=c(0,0), topLabSize=c(0,0.5),
        colsep=2:ncol(dfStars), rowsep=2:nrow(dfStars), sepcolor="gray", sepwidth=c(0.025,0.05),
        density.info="none", cexRow=0.6, cexCol=0.8, cexLabKey=0.6, cex.main=1.2,
        valueForNA=-4, heatmapColors=colors, breaks=breaks, rowDendroSize=c(90,10),
        topLabCol=FALSE, btmLabRow=TRUE, lftLabRow=TRUE, rgtLabRow=FALSE, srtLabCol=45,
        titleRowDendrogram="ILs clustered by abs. Pearson corr. of each trait's p-value for comparison to M82",
        titleHeatmap="Heatmap: ILs with significant p-value in trait comparison to M82\nILs clustered by abs. Pearson corr. of p-value",
        subtitleHeatmap=colorSet$subtitle)

    # Make subset heatmaps.
    for (subsetGTs in makeHeatmapSubsetGTs)
        {
        dfSubset = dfStars[subsetGTs,]
        makeHeatmap(dfSubset, colDendro=NULL, rowDendro=NULL, xlabKey=xlabKey, xvalKey=xvalKey,
            margins=c(bottom=50,left=30,top=10,right=35),
            heatmapAreaSize=c(35,30), heatmapSize=c(50,20), keySize=c(90,20), keySpace=c(10,30), key=3,
            colsep=2:ncol(dfSubset), rowsep=2:nrow(dfSubset), sepcolor="gray", sepwidth=c(0.01,0.02),
            density.info="none", cexRow=0.6, cexCol=0.8, cexLabKey=0.6, cex.main=1.2,
            valueForNA=-4, heatmapColors=colors, breaks=breaks,
            topLabCol=TRUE, btmLabCol=FALSE,
            lftLabRow=TRUE, rgtLabRow=FALSE, lftLabSize=c(30,0.5), srtLabCol=45,
            titleHeatmap="Heatmap: subset of ILs with significant p-value in trait comparison to M82",
            subtitleHeatmap=colorSet$subtitle)
        }

    # Close the pdf file.
    dev.off()
    }

################################################################################
# End.
################################################################################
