#######################################################################################
# This file contains R definitions and functions relating to making of Venn Diagrams.
#
# To use this file:
#   1. Copy it from the dropbox to a local directory, so changes to it don't mess you up.
#   2. Add a "source" line to the start of .R files in which you want to use this file,
#       to source either this file, or more preferably, file "Include_UtilityFunctions.R"
#       (which includes this file):
#           source("Include_UtilityFunctions.R")
#   3. Call any functions you want that are in here, or use any constants from here.
#      Skim through the file to see what is available.
#
# Revision history:
# Author    Date        Description
# --------  -------     ---------------------------------------------------------------
# Ted Toal  04Mar12     Add this file to BradyLab dropbox to share with lab members.
# Ted Toal  04May12     Add vennPlot().
#######################################################################################

#######################################################################################
# Plot a Venn Diagram of 1 to 4 sets.
# This is venn.diagram() of package VennDiagram, modified as follows by Ted Toal:
#   New arguments:
#       labels: labels to use for circles; default is to use names of x.
#   Add argument antialias="none" to tiff() call.
#######################################################################################
tedVenn = function(x, filename=NULL, height=3000, width=3000, resolution=500, 
    units="px", compression="lzw", na="stop", main="", 
    sub="", main.pos=c(0.5, 1.05), main.fontface="plain", 
    main.fontfamily="serif", main.col="black", main.cex=1, 
    main.just=c(0.5, 1), sub.pos=c(0.5, 1.05), sub.fontface="plain", 
    sub.fontfamily="serif", sub.col="black", sub.cex=1, 
    sub.just=c(0.5, 1), labels=names(x), fill=c("orange", "red2", "blue", "lightgreen"), ...) 
    {
    options(warn = -1)
    if (na == "none") {
        x = x
    }
    if (na == "stop") {
        for (i in 1:length(x)) {
            if (length(x[[i]][is.na(x[[i]])]) > 0) {
                stop("NAs in dataset", call. = FALSE)
            }
        }
    }
    if (na == "remove") {
        for (i in length(x)) {
            x[[i]] = x[[i]][!is.na(x[[i]])]
        }
    }
    if (length(x) == 0 | length(x) > 4) {
        stop("Incorrect number of elements.", call. = FALSE)
    }
    if (length(fill) > length(x)) {
        fill = fill[1:length(x)]
    }
    if (length(x) == 1) {
        list.names = labels
        if (is.null(list.names)) {
            list.names = ""
        }
        grob.list = draw.single.venn(length(x[[1]]), list.names, 
            ind = FALSE, fill=fill, ...)
        if (sub != "") {
            grob.list = add.title(gList = grob.list, x = sub, 
                pos = sub.pos, fontface = sub.fontface, fontfamily = sub.fontfamily, 
                col = sub.col, cex = sub.cex)
        }
        if (main != "") {
            grob.list = add.title(gList = grob.list, x = main, 
                pos = main.pos, fontface = main.fontface, fontfamily = main.fontfamily, 
                col = main.col, cex = main.cex)
        }
    }
    else if (length(x) == 2) {
        a = x[[1]]
        b = x[[2]]
        intersect = intersect(a, b)
        list.names = c(labels[1], labels[2])
        if (length(a) < length(b)) {
            list.names = rev(list.names)
        }
        grob.list = draw.pairwise.venn(length(a), length(b), 
            length(intersect), list.names, ind = FALSE, fill=fill, ...)
        if (sub != "") {
            grob.list = add.title(gList = grob.list, x = sub, 
                pos = sub.pos, fontface = sub.fontface, fontfamily = sub.fontfamily, 
                col = sub.col, cex = sub.cex)
        }
        if (main != "") {
            grob.list = add.title(gList = grob.list, x = main, 
                pos = main.pos, fontface = main.fontface, fontfamily = main.fontfamily, 
                col = main.col, cex = main.cex)
        }
    }
    else if (length(x) == 3) {
        a = x[[1]]
        b = x[[2]]
        c = x[[3]]
        list.names = c(labels[1], labels[2], labels[3])
        nab = intersect(a, b)
        nbc = intersect(b, c)
        nac = intersect(a, c)
        nabc = intersect(nab, c)
        grob.list = draw.triple.venn(length(a), length(b), length(c), 
            length(nab), length(nbc), length(nac), length(nabc), 
            list.names, ind = FALSE, list.order = 1:3, fill=fill, ...)
        if (sub != "") {
            grob.list = add.title(gList = grob.list, x = sub, 
                pos = sub.pos, fontface = sub.fontface, fontfamily = sub.fontfamily, 
                col = sub.col, cex = sub.cex)
        }
        if (main != "") {
            grob.list = add.title(gList = grob.list, x = main, 
                pos = main.pos, fontface = main.fontface, fontfamily = main.fontfamily, 
                col = main.col, cex = main.cex)
        }
    }
    else if (length(x) == 4) {
        a = x[[1]]
        b = x[[2]]
        c = x[[3]]
        d = x[[4]]
        list.names = c(labels[1], labels[2], labels[3], labels[4])
        n12 = intersect(a, b)
        n13 = intersect(a, c)
        n14 = intersect(a, d)
        n23 = intersect(b, c)
        n24 = intersect(b, d)
        n34 = intersect(c, d)
        n123 = intersect(n12, c)
        n124 = intersect(n12, d)
        n134 = intersect(n13, d)
        n234 = intersect(n23, d)
        n1234 = intersect(n123, d)
        grob.list = draw.quad.venn(area1 = length(a), area2 = length(b), 
            area3 = length(c), area4 = length(d), n12 = length(n12), 
            n13 = length(n13), n14 = length(n14), n23 = length(n23), 
            n24 = length(n24), n34 = length(n34), n123 = length(n123), 
            n124 = length(n124), n134 = length(n134), n234 = length(n234), 
            n1234 = length(n1234), category = list.names, ind = FALSE, 
            fill=fill, ...)
        if (sub != "") {
            grob.list = add.title(gList = grob.list, x = sub, 
                pos = sub.pos, fontface = sub.fontface, fontfamily = sub.fontfamily, 
                col = sub.col, cex = sub.cex)
        }
        if (main != "") {
            grob.list = add.title(gList = grob.list, x = main, 
                pos = main.pos, fontface = main.fontface, fontfamily = main.fontfamily, 
                col = main.col, cex = main.cex)
        }
    }
    else {
        stop("Invalid size of input object")
    }
    if (!is.null(filename)) {
        tiff(filename = filename, height = height, width = width, antialias = "none",
            units = units, res = resolution, compression = compression)
        grid.draw(grob.list)
        dev.off()
        return(1)
    }
    return(grob.list)
    }

#######################################################################################
# Modified version of function adjust.venn() of package VennDiagram.  We call this to
# attempt to move the diagram up on the page.
#######################################################################################
adjustVenn = function(gList1, mgns=rep(0.1,4), plotSize=0.90, ...) 
    {
    x.vect <- vector()
    y.vect <- vector()
    x.list <- list()
    y.list <- list()
    for (i in 1:length(gList1))
        {
        x.vect <- c(x.vect, as.vector(gList1[i][[1]]$x, mode = "numeric"))
        y.vect <- c(y.vect, as.vector(gList1[i][[1]]$y, mode = "numeric"))
        x.list[[i]] <- as.vector(gList1[i][[1]]$x, mode = "numeric")
        y.list[[i]] <- as.vector(gList1[i][[1]]$y, mode = "numeric")
        }
    max.x <- max(x.vect) + mgns[4]
    min.x <- min(x.vect) - mgns[2]
    max.y <- max(y.vect) + mgns[3]
    min.y <- min(y.vect) - mgns[1]
    x.centre <- (max.x + min.x)/2
    y.centre <- (max.y + min.y)/2
    size <- plotSize
    if (max.x - min.x >= max.y - min.y)
        {
        for (i in 1:length(x.list))
            {
            x.list[[i]] <- unit((x.list[[i]] - x.centre) * (size/(max.x - 
                min.x)) + 0.5, "npc")
            y.list[[i]] <- unit((y.list[[i]] - y.centre) * (size/(max.x - 
                min.x)) + 0.5, "npc")
            }
        }
    if (max.x - min.x < max.y - min.y)
        {
        for (i in 1:length(x.list))
            {
            x.list[[i]] <- unit((x.list[[i]] - x.centre) * (size/(max.y - 
                min.y)) + 0.5, "npc")
            y.list[[i]] <- unit((y.list[[i]] - y.centre) * (size/(max.y - 
                min.y)) + 0.5, "npc")
            }
        }
    for (i in 1:length(gList1))
        {
        gList1[i][[1]]$x <- x.list[[i]]
        gList1[i][[1]]$y <- y.list[[i]]
        }
    return(gList1)
    }

#######################################################################################
# Use the current graphical device to plot Venn diagrams using function venn.diagram()
# of package VennDiagram.  See that function for description of arguments.
#######################################################################################
vennPlot = function(x, na="stop", main="", 
    sub="", main.pos=c(0.5, 1.05), main.fontface="plain", 
    main.fontfamily="serif", main.col="black", main.cex=1, 
    main.just=c(0.5, 1), sub.pos=c(0.5, 1.05), sub.fontface="plain", 
    sub.fontfamily="serif", sub.col="black", sub.cex=1, 
    sub.just=c(0.5, 1), labels=names(x), fill=c("orange", "red2", "blue", "lightgreen"),
    mgns=c(0.1,0.1,0.1,0.1), plotSize=0.90, ...) 
    {
    plot.new()
    grobList = tedVenn(x, NULL, 0, 0, 0, "", "",
        na, main, sub, main.pos, main.fontface, main.fontfamily, main.col, main.cex, 
        main.just, sub.pos, sub.fontface, sub.fontfamily, sub.col, sub.cex, sub.just,
        labels, fill, ...) 
    grobList = adjustVenn(grobList, mgns, plotSize)
    grid.draw(grobList)
    }

#######################################################################################
# Plot a Venn Diagram of 5 sets.
# This is plot_venn5() courtesy of:
#   Liqun He
#   Karolinska Institute, Stockholm
# Modified by Ted Toal (add membersIfLt and memberSep).
# New arguments:
#   vsets: first argument is a list of five vectors/lists of set members.
#   labels: optional list of set labels
#   cex.labels, col.labels, vfont.labels, font.labels: cex, col, vfont, and font
#       values for plotting set labels
#   cex.sizes, col.sizes, vfont.sizes, font.sizes: 5-vectors of cex, col, vfont,
#       and font values for plotting set sizes.  The index is 1 for regions
#       containing only 1 set, 2 for regions containing 2 sets, etc.
#   e.density, e.angle, e.col, e.border, e.lty, e.lwd: 5-vectors of density, angle,
#       col, border, lty, and lwd values for plotting ellipses
#   alpha: alpha value for ellipse fill, 0-1.
#   elementsIfLt: size of an intersection region such that if a region's size is
#       less than this, the element names are displayed in the region rather than
#       the size of the region.  Default of 0 means size of region always displayed.
#   elementSep: string to separate element names if they are displayed in a region.
#       This might be " " (space), "," (comma), or "\n" (newline, each on new line).
#   cexSizeMult: factor to multiply all cex values for region size/element names.
#       This scales all region size/element name cex values simultaneously.  Use
#       cex.sizes for finer control of scaling of individual regions.
#   cexShrinkPerElement: factor to multiply cex for element name text.  If N element
#       names are displayed in a region, the cex is multiplied by this value ^ (N-1).
#   cexMin: minimum allowed value for a cex value for display of element names.  If
#       cexSizeMult and cexShrinkPerElement shrink it smaller than this, it is
#       clamped at this.
#   cexMax: maximum allowed value for a cex value for display of element names.  If
#       cexSizeMult and cexShrinkPerElement result in it being larger than this, it is
#       clamped at this.
#   suppress0: TRUE to suppress 0 labels when set region size is 0.
#   showSetSize: TRUE to show set size after set label.
#   kShowPt: a number from 0 to 5.  If > 0, a point is drawn within each region that
#       is common to showX or more sets.  The position at which the point is drawn
#       is the center point of the element name or set size text.  The reason you
#       might want to plot points is that there might be so many element names that
#       they spill off outside the region and it is hard to identify where the
#       region is.
#   pch.pt: when showPt > 0, this is the pch value of the point (see points).
#   cex.pt: when showPt > 0, this is the cex value of the point.
#   col.pt: when showPt > 0, this is the color of the point.
#   axes, xlab, ylab, and ...: passed to plot().
#   nudgeSetLabels: either NULL or a list indicating how much to move the default position
#       of set labels.  The list element names are set names A-E, and the element values are
#       2-element vectors giving the delta value for the x- and y- position of the set label.
#       The entire diagram is plotted on a 400 x 400 grid, so that gives the scale of the
#       values that might be needed.  The list need not contain all 5 names.
#   nudgeIntersectionLabels: either NULL or a list indicating how much to move the default
#       position of set intersection labels.  The list element names are set intersection
#       names using A-E as the five set names, so that a list element name of ACD means the
#       label for the region representing the intersection of sets A, C, and D.  The list
#       element values are 2-element vectors just as for nudgeSetLabels.  Note that set A is
#       the one at 10 o'clock and they move around clockwise.
#   testIsectLabelPositioning: if TRUE, just plot a dot in each intersection, with its set names.
# Example:
#   vsets = list(L1=letters[1:6], L2=letters[6:10], L3=letters[6:15], L4=letters[6:21], L5=letters[6:26])
#   plotVenn5(vsets, main="Hello World", showSetSize=TRUE)
#######################################################################################
plotVenn5 = function(vsets,
    labels=names(vsets), cex.labels=2.0, col.labels=NULL, vfont.labels=NULL, font.labels=NULL,
    cex.sizes=c(2.0, 0.7, 0.7, 0.7, 0.6), col.sizes=rep(NULL,5), vfont.sizes=rep(NULL,5), font.sizes=rep(NULL,5),
    e.density=rep(NULL,5), e.angle=rep(45,5), e.col=c("red", "blue", "green", "pink", "yellow"),
    e.border=rep(NULL,5), e.lty=rep(par("lty"),5), e.lwd=rep(par("lwd"),5), alpha=0.4,
    elementsIfLt=0, elementSep="\n", cexSizeMult=1.0, cexShrinkPerElement=0.95, cexMin=0.6, cexMax=3.0,
    suppress0=TRUE, showSetSize=FALSE, kShowPt=0, pch.pt=20, cex.pt=0.9, col.pt="black",
    axes=FALSE, ylab="", xlab="", nudgeSetLabels=NULL, nudgeIntersectionLabels=NULL,
    testIsectLabelPositioning=FALSE, ...)
    {
    if (length(vsets) != 5) stop("plotVenn5 requires first argument to be a list of five set members.")
    all_ids = unique(unlist(vsets))
    all_n = length(all_ids)
    
    # Create counting matrix.

    all_matrix = cbind(L1=rep(F, all_n), L2=rep(F, all_n), L3=rep(F, all_n), L4=rep(F, all_n), L5=rep(F, all_n))
    rownames(all_matrix) = all_ids
    colnames(all_matrix) = names(vsets)
    all_matrix[all_ids %in% vsets[[1]], 1] = T
    all_matrix[all_ids %in% vsets[[2]], 2] = T
    all_matrix[all_ids %in% vsets[[3]], 3] = T
    all_matrix[all_ids %in% vsets[[4]], 4] = T
    all_matrix[all_ids %in% vsets[[5]], 5] = T
    
    # Elements only in one data set.  eX = vector of elements in intersection region X.
    
    only1 = apply(all_matrix, 1, sum) == 1
    eA = all_ids[only1][all_matrix[only1,1]]
    eB = all_ids[only1][all_matrix[only1,2]]
    eC = all_ids[only1][all_matrix[only1,3]]
    eD = all_ids[only1][all_matrix[only1,4]]
    eE = all_ids[only1][all_matrix[only1,5]]
    
    # Elements overlapped in two data sets.
    
    only2 = apply(all_matrix, 1, sum) == 2
    eAB = all_ids[only2][all_matrix[only2,1] & all_matrix[only2,2]]
    eAC = all_ids[only2][all_matrix[only2,1] & all_matrix[only2,3]]
    eAD = all_ids[only2][all_matrix[only2,1] & all_matrix[only2,4]]
    eAE = all_ids[only2][all_matrix[only2,1] & all_matrix[only2,5]]
    eBC = all_ids[only2][all_matrix[only2,2] & all_matrix[only2,3]]
    eBD = all_ids[only2][all_matrix[only2,2] & all_matrix[only2,4]]
    eBE = all_ids[only2][all_matrix[only2,2] & all_matrix[only2,5]]
    eCD = all_ids[only2][all_matrix[only2,3] & all_matrix[only2,4]]
    eCE = all_ids[only2][all_matrix[only2,3] & all_matrix[only2,5]]
    eDE = all_ids[only2][all_matrix[only2,4] & all_matrix[only2,5]]
    
    # Elements overlapped in three data sets.
    
    only3 = apply(all_matrix, 1, sum) == 3
    eABC = all_ids[only3][all_matrix[only3,1] & all_matrix[only3,2] & all_matrix[only3,3]]
    eABD = all_ids[only3][all_matrix[only3,1] & all_matrix[only3,2] & all_matrix[only3,4]]
    eABE = all_ids[only3][all_matrix[only3,1] & all_matrix[only3,2] & all_matrix[only3,5]]
    eACD = all_ids[only3][all_matrix[only3,1] & all_matrix[only3,3] & all_matrix[only3,4]]
    eACE = all_ids[only3][all_matrix[only3,1] & all_matrix[only3,3] & all_matrix[only3,5]]
    eADE = all_ids[only3][all_matrix[only3,1] & all_matrix[only3,4] & all_matrix[only3,5]]
    eBCD = all_ids[only3][all_matrix[only3,2] & all_matrix[only3,3] & all_matrix[only3,4]]
    eBCE = all_ids[only3][all_matrix[only3,2] & all_matrix[only3,3] & all_matrix[only3,5]]
    eBDE = all_ids[only3][all_matrix[only3,2] & all_matrix[only3,4] & all_matrix[only3,5]]
    eCDE = all_ids[only3][all_matrix[only3,3] & all_matrix[only3,4] & all_matrix[only3,5]]
    
    # Elements overlapped in four data sets.
    
    only4 = apply(all_matrix, 1, sum) == 4
    eABCD = all_ids[only4][all_matrix[only4,1] & all_matrix[only4,2] & all_matrix[only4,3] & all_matrix[only4,4]]
    eABCE = all_ids[only4][all_matrix[only4,1] & all_matrix[only4,2] & all_matrix[only4,3] & all_matrix[only4,5]]
    eABDE = all_ids[only4][all_matrix[only4,1] & all_matrix[only4,2] & all_matrix[only4,4] & all_matrix[only4,5]]
    eACDE = all_ids[only4][all_matrix[only4,1] & all_matrix[only4,3] & all_matrix[only4,4] & all_matrix[only4,5]]
    eBCDE = all_ids[only4][all_matrix[only4,2] & all_matrix[only4,3] & all_matrix[only4,4] & all_matrix[only4,5]]
    
    # Elements overlapped in five data sets.
    
    all5 = apply(all_matrix, 1, sum) == 5
    eABCDE = all_ids[all5]
    
    # Make the plot.
    
    elps = cbind(150*cos(seq(0, 2*pi, len=1000)), 60*sin(seq(0, 2*pi, len=1000)))
    
    relocate_elp = function(e, alpha, x, y)
        {
        phi = (alpha/180)*pi
        xr = e[,1]*cos(phi)+e[,2]*sin(phi)
        yr = -e[,1]*sin(phi)+e[,2]*cos(phi)
        xr = x+xr
        yr = y+yr
        return(cbind(xr, yr))
        }
    
    par(mar=c(1, 1, 1, 1))
    plot(c(0, 400), c(0, 400), type="n", axes=axes, ylab=ylab, xlab=xlab, ...)
    
    col = col2rgb(e.col)
    col = rgb(col["red",], col["green",], col["blue",], alpha*255, maxColorValue=255)
    polygon(relocate_elp(elps, 90, 200, 250), density=e.density[1], angle=e.angle[1], col=col[1], border=e.border[1], lty=e.lty[1], lwd=e.lwd[1])
    polygon(relocate_elp(elps, 162, 250, 220), density=e.density[2], angle=e.angle[2], col=col[2], border=e.border[2], lty=e.lty[2], lwd=e.lwd[2])
    polygon(relocate_elp(elps, 234, 250, 150), density=e.density[3], angle=e.angle[3], col=col[3], border=e.border[3], lty=e.lty[3], lwd=e.lwd[3])
    polygon(relocate_elp(elps, 306, 180, 125), density=e.density[4], angle=e.angle[4], col=col[4], border=e.border[4], lty=e.lty[4], lwd=e.lwd[4])
    polygon(relocate_elp(elps, 378, 145, 200), density=e.density[5], angle=e.angle[5], col=col[5], border=e.border[5], lty=e.lty[5], lwd=e.lwd[5])
    
    # Default positions for set labels.

    posSetLabels=list(A=c(50, 280), B=c(170, 400), C=c(350, 300), D=c(350, 20), E=c(55, 10))

    # Label the data set names.  'k' is the set number 1-5.

    putSetLabel = function(k)
        {
        label = labels[k]
        if (showSetSize)
            label = paste(label, " [", length(vsets[[k]]), "]", sep="")
        pos = posSetLabels[[k]]
        if (!is.null(nudgeSetLabels) && (A:E)[i] %in% names(nudgeSetLabels))
            pos = pos + nudgeSetLabels[[(A:E)[i]]]
        x = pos[1]
        y = pos[2]
        text(x, y, label, cex=cex.labels, col=col.labels, vfont=vfont.labels, font=font.labels)
        }

    putSetLabel(1)
    putSetLabel(2)
    putSetLabel(3)
    putSetLabel(4)
    putSetLabel(5)

    # Default positions for set intersection labels.

    posIntersectionLabels=list(
        A=c(61, 228),B=c(194, 329),C=c(321, 245),D=c(290, 81),E=c(132, 69),
            AB=c(146, 250),AC=c(123, 188),AD=c(275, 152),AE=c(137, 146),
            BC=c(243, 268),BD=c(175, 267),BE=c(187, 117),
            CD=c(286, 188),CE=c(267, 235),DE=c(228, 105),
                ABC=c(148, 210),ABD=c(159, 253),ABE=c(171, 141),
                ACD=c(281, 175),ACE=c(143, 163),ADE=c(252, 145),
                BCD=c(205, 255),BCE=c(254, 243),BDE=c(211, 118),CDE=c(267, 211),
                    ABCD=c(170, 231),ABCE=c(158, 169),ABDE=c(212, 139),ACDE=c(263, 180),BCDE=c(239, 232),
                        ABCDE=c(204, 190))

    # Plot the element names or number of elements in a set intersection region,
    # with an optional X depending on kShowPt.  'sets' is a string identifying
    # the sets in the intersection (a name in the deltaIntersectionLabels list names),
    # 'eX' is a vector of elements in the intersecting region, and 'k' is the number
    # of intersecting sets in the region.

    plotIntersectionLabel = function(sets, eX, k)
        {
        pos = posIntersectionLabels[[sets]]
        if (!is.null(nudgeIntersectionLabels) && sets %in% names(nudgeIntersectionLabels))
            pos = pos + nudgeIntersectionLabels[[sets]]
        x = pos[1]
        y = pos[2]
        if (testIsectLabelPositioning)
            {
            text(x, y, sets, cex=cex.sizes[k], col=col.sizes[k], vfont=vfont.sizes[k], font=font.sizes[k], pos=2)
            points(x, y, pch=pch.pt, cex=cex.pt, col=col.pt)
            return(invisible(0))
            }
        N = length(eX)
        if (!suppress0 || N != 0)
            {
            cex = cex.sizes[k] * cexSizeMult
            adjPt.y = -0.75
            if (N < elementsIfLt)
                {
                S = paste(eX, collapse=elementSep)
                cex = cex*(cexShrinkPerElement^(N-1))
                if (length(eX) %% 2 == 0)
                    adjPt.y = -0.1 # Try to make X between lines.
                }
            else
                S = as.character(N)

            if (cex < cexMin)
                cex = cexMin
            if (cex > cexMax)
                cex = cexMax

            text(x, y, S, cex=cex, col=col.sizes[k], vfont=vfont.sizes[k], font=font.sizes[k], adj=c(0.5,0.5))
            if (kShowPt > 0 && k >= kShowPt)
                {
                H = strheight("X", cex=cex, vfont=vfont.sizes[k], font=font.sizes[k])
                points(x, y+H*adjPt.y, pch=pch.pt, cex=cex.pt, col=col.pt)
                }
            }
        }

    plotIntersectionLabel("A", eA, 1)
    plotIntersectionLabel("B", eB, 1)
    plotIntersectionLabel("C", eC, 1)
    plotIntersectionLabel("D", eD, 1)
    plotIntersectionLabel("E", eE, 1)

    plotIntersectionLabel("AB", eAB, 2)
    plotIntersectionLabel("AC", eAC, 2)
    plotIntersectionLabel("AD", eAD, 2)
    plotIntersectionLabel("AE", eAE, 2)
    plotIntersectionLabel("BC", eBC, 2)
    plotIntersectionLabel("BD", eBD, 2)
    plotIntersectionLabel("BE", eBE, 2)
    plotIntersectionLabel("CD", eCD, 2)
    plotIntersectionLabel("CE", eCE, 2)
    plotIntersectionLabel("DE", eDE, 2)

    plotIntersectionLabel("ABC", eABC, 3)
    plotIntersectionLabel("ABD", eABD, 3)
    plotIntersectionLabel("ABE", eABE, 3)
    plotIntersectionLabel("ACD", eACD, 3)
    plotIntersectionLabel("ACE", eACE, 3)
    plotIntersectionLabel("ADE", eADE, 3)
    plotIntersectionLabel("BCD", eBCD, 3)
    plotIntersectionLabel("BCE", eBCE, 3)
    plotIntersectionLabel("BDE", eBDE, 3)
    plotIntersectionLabel("CDE", eCDE, 3)

    plotIntersectionLabel("ABCD", eABCD, 4)
    plotIntersectionLabel("ABCE", eABCE, 4)
    plotIntersectionLabel("ABDE", eABDE, 4)
    plotIntersectionLabel("ACDE", eACDE, 4)
    plotIntersectionLabel("BCDE", eBCDE, 4)

    plotIntersectionLabel("ABCDE", eABCDE, 5)
    }

#######################################################################################
# End of file.
#######################################################################################
cat("Include_VennDiagrams.R included\n")
